# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os

from awscrt.s3 import S3Request
from botocore.awsrequest import AWSResponse

from awscli.clidriver import AWSCLIEntryPoint
from awscli.compat import BytesIO, urlparse
from awscli.testutils import (
    BaseAWSCommandParamsTest,
    FileCreator,
    create_clidriver,
    mock,
    temporary_file,
    unittest,
)
from tests import CLIRunner, HTTPResponse, SessionStubber


class BaseS3TransferCommandTest(BaseAWSCommandParamsTest):
    def setUp(self):
        super(BaseS3TransferCommandTest, self).setUp()
        self.files = FileCreator()
        self.init_clidriver()

    def init_clidriver(self):
        with temporary_file('w') as f:
            f.write('[default]\n' 's3 =\n' '  max_concurrent_requests = 1\n')
            f.flush()
            self.environ['AWS_CONFIG_FILE'] = f.name
            self.driver = create_clidriver()
            self.entry_point = AWSCLIEntryPoint(self.driver)

    def tearDown(self):
        super(BaseS3TransferCommandTest, self).tearDown()
        self.files.remove_all()

    def assert_operations_called(self, expected_operations_with_params):
        actual_operations_with_params = [
            (operation_called[0].name, operation_called[1])
            for operation_called in self.operations_called
        ]
        self.assertEqual(
            actual_operations_with_params, expected_operations_with_params
        )

    def assert_in_operations_called(self, expected_operation_with_params):
        actual_operations_with_params = [
            (operation_called[0].name, operation_called[1])
            for operation_called in self.operations_called
        ]
        for actual_operation_with_params in actual_operations_with_params:
            if expected_operation_with_params == actual_operation_with_params:
                return
        self.fail(
            'Expected request: %s does not match any of the actual requests '
            'made: %s'
            % (expected_operation_with_params, actual_operations_with_params)
        )

    def head_object_response(self, **override_kwargs):
        response = {
            'ContentLength': 100,
            'LastModified': '00:00:00Z',
            'ETag': '"foo-1"',
        }
        response.update(override_kwargs)
        return response

    def list_objects_response(self, keys, **override_kwargs):
        contents = []
        for key in keys:
            content = {
                'Key': key,
                'LastModified': '00:00:00Z',
                'Size': 100,
                'ETag': '"foo-1"',
            }
            if override_kwargs:
                content.update(override_kwargs)
            contents.append(content)
        return {'Contents': contents, 'CommonPrefixes': []}

    def get_object_response(self):
        return {'ETag': '"foo-1"', 'Body': BytesIO(b'foo')}

    def copy_object_response(self):
        return self.empty_response()

    def delete_object_response(self):
        return self.empty_response()

    def create_mpu_response(self, upload_id):
        return {'UploadId': upload_id}

    def upload_part_copy_response(self):
        return {'CopyPartResult': {'ETag': '"etag"'}}

    def complete_mpu_response(self):
        return self.empty_response()

    def get_object_tagging_response(self, tags):
        return {'TagSet': [{'Key': k, 'Value': v} for k, v in tags.items()]}

    def put_object_tagging_response(self):
        return 'PutObjectTagging', self.empty_response()

    def empty_response(self):
        return {}

    def head_object_request(self, bucket, key, **override_kwargs):
        params = {
            'Bucket': bucket,
            'Key': key,
        }
        params.update(override_kwargs)
        return 'HeadObject', params

    def list_objects_request(self, bucket, prefix=None, **override_kwargs):
        params = {
            'Bucket': bucket,
        }
        if prefix is None:
            params['Prefix'] = ''
        params.update(override_kwargs)
        return 'ListObjectsV2', params

    def put_object_request(self, bucket, key, **override_kwargs):
        params = {
            'Bucket': bucket,
            'Key': key,
            'ChecksumAlgorithm': 'CRC64NVME',
            'Body': mock.ANY,
        }
        params.update(override_kwargs)
        return 'PutObject', params

    def get_object_request(self, bucket, key, **override_kwargs):
        params = {
            'Bucket': bucket,
            'Key': key,
        }
        params.update(override_kwargs)
        return 'GetObject', params

    def copy_object_request(
        self, source_bucket, source_key, bucket, key, **override_kwargs
    ):
        params = {
            'Bucket': bucket,
            'Key': key,
            'CopySource': {'Bucket': source_bucket, 'Key': source_key},
        }
        params.update(override_kwargs)
        return 'CopyObject', params

    def delete_object_request(self, bucket, key, **override_kwargs):
        params = {
            'Bucket': bucket,
            'Key': key,
        }
        params.update(override_kwargs)
        return 'DeleteObject', params

    def create_mpu_request(self, bucket, key, **override_kwargs):
        params = {
            'Bucket': bucket,
            'Key': key,
        }
        params.update(override_kwargs)
        return 'CreateMultipartUpload', params

    def upload_part_copy_request(
        self,
        source_bucket,
        source_key,
        bucket,
        key,
        upload_id,
        **override_kwargs,
    ):
        params = {
            'Bucket': bucket,
            'Key': key,
            'CopySource': {'Bucket': source_bucket, 'Key': source_key},
            'UploadId': upload_id,
        }
        params.update(override_kwargs)
        return 'UploadPartCopy', params

    def complete_mpu_request(
        self, bucket, key, upload_id, num_parts, **override_kwargs
    ):
        parts = []
        for i in range(num_parts):
            parts.append({'ETag': '"etag"', 'PartNumber': i + 1})
        params = {
            'Bucket': bucket,
            'Key': key,
            'UploadId': upload_id,
            'MultipartUpload': {'Parts': parts},
        }
        params.update(override_kwargs)
        return 'CompleteMultipartUpload', params

    def get_object_tagging_request(self, bucket, key):
        return 'GetObjectTagging', {
            'Bucket': bucket,
            'Key': key,
        }

    def put_object_tagging_request(self, bucket, key, tags):
        return 'PutObjectTagging', {
            'Bucket': bucket,
            'Key': key,
            'Tagging': {
                'TagSet': [{'Key': k, 'Value': v} for k, v in tags.items()],
            },
        }

    def no_such_key_error_response(self):
        return {
            'Error': {
                'Code': 'NoSuchKey',
                'Message': 'The specified key does not exist',
            }
        }

    def access_denied_error_response(self):
        return {
            'Error': {
                'Code': 'AccessDenied',
                'Message': 'Operation not allowed',
            }
        }

    def set_http_status_codes(self, status_codes):
        self.http_responses = [
            AWSResponse(None, code, {}, None) for code in status_codes
        ]

    def mp_copy_responses(self):
        return [
            self.create_mpu_response('upload_id'),
            self.upload_part_copy_response(),
            self.complete_mpu_response(),
        ]

    def precondition_failed_error_response(self, condition='If-None-Match'):
        return {
            'Error': {
                'Code': 'PreconditionFailed',
                'Message': 'At least one of the pre-conditions you '
                'specified did not hold',
                'Condition': condition,
            }
        }


class BaseS3CLIRunnerTest(unittest.TestCase):
    def setUp(self):
        self.session_stubber = self.get_session_stubber()
        self.cli_runner = CLIRunner(session_stubber=self.session_stubber)

        self.region = 'us-west-2'
        self.cli_runner.env['AWS_DEFAULT_REGION'] = self.region

        self.config_files = FileCreator()
        self.config_filename = os.path.join(
            self.config_files.rootdir, 'config'
        )
        self.set_config_file_contents(
            self.cli_runner.env, self.config_filename
        )

    def tearDown(self):
        self.config_files.remove_all()

    def get_session_stubber(self):
        return SessionStubber()

    def set_config_file_contents(self, env, config_filename):
        with open(config_filename, 'w') as f:
            f.write(self.get_config_file_contents())
            f.flush()
        env['AWS_CONFIG_FILE'] = config_filename

    def get_config_file_contents(self):
        return (
            '[default]\n'
            's3 =\n'
            '  preferred_transfer_client = classic\n'
            '  max_concurrent_requests = 1\n'
        )

    def get_virtual_s3_host(self, bucket, region=None):
        if not region:
            region = self.region
        return f'{bucket}.s3.{region}.amazonaws.com'

    def add_botocore_head_object_response(self, size=100):
        self.cli_runner.add_response(
            HTTPResponse(
                headers={
                    'Content-Length': str(size),
                    'Last-Modified': 'Thu, 11 Feb 2021 04:24:23 GMT',
                    'ETag': 'foobar',
                }
            )
        )

    def add_botocore_list_objects_response(self, keys, size=100):
        xml_body = (
            '<?xml version="1.0" ?>'
            '<ListBucketResult xmlns='
            '"http://s3.amazonaws.com/doc/2006-03-01/">'
            '<Prefix/>'
        )
        for key in keys:
            xml_body += (
                '<Contents>'
                '<LastModified>2015-12-08T18:26:43.000Z</LastModified>'
                f'<Key>{key}</Key>'
                f'<Size>{size}</Size>'
                f'<ETag>foobar</ETag>'
                '</Contents>'
            )
        xml_body += '</ListBucketResult>'
        self.cli_runner.add_response(HTTPResponse(body=xml_body))

    def add_botocore_put_object_response(self):
        self.cli_runner.add_response(HTTPResponse())

    def add_botocore_get_object_response(self):
        self.cli_runner.add_response(
            HTTPResponse(headers={'ETag': '"foo-1"'}, body=BytesIO(b'foo'))
        )

    def add_botocore_copy_object_response(self):
        self.cli_runner.add_response(
            HTTPResponse(body='<CopyObjectResult></CopyObjectResult>')
        )

    def add_botocore_delete_object_response(self):
        self.cli_runner.add_response(HTTPResponse())

    def add_botocore_create_multipart_upload_response(self):
        self.cli_runner.add_response(
            HTTPResponse(
                body=(
                    '<InitiateMultipartUploadResult>'
                    '  <Bucket>bucket</Bucket>'
                    '  <Key>key</Key>'
                    '  <UploadId>upload-id</UploadId>'
                    '</InitiateMultipartUploadResult>'
                )
            )
        )

    def add_botocore_upload_part_copy_response(self):
        self.cli_runner.add_response(
            HTTPResponse(
                body=(
                    '<CopyPartResult>'
                    '  <LastModified>2022-09-30T17:20:13.000Z</LastModified>'
                    '  <ETag>&quot;etag&quot;</ETag>'
                    '</CopyPartResult>'
                )
            )
        )

    def add_botocore_complete_multipart_upload_response(self):
        self.cli_runner.add_response(
            HTTPResponse(
                body=(
                    '<CompleteMultipartUploadResult>'
                    '</CompleteMultipartUploadResult>'
                )
            )
        )

    def add_botocore_get_object_tagging_response(self, tags=None):
        tag_set_xml = '<TagSet>'
        if tags:
            for k, v in tags.items():
                tag_set_xml += f'<Tag><Key>{k}</Key><Value>{v}</Value></Tag>'
        tag_set_xml += '</TagSet>'
        self.cli_runner.add_response(
            HTTPResponse(body=f'<Tagging>{tag_set_xml}</Tagging>')
        )

    def add_botocore_set_object_tagging_response(self):
        self.cli_runner.add_response(HTTPResponse())

    def assert_no_remaining_botocore_responses(self):
        self.session_stubber.assert_no_remaining_responses()

    def assert_operations_to_endpoints(
        self, cli_runner_result, expected_operations_to_endpoints
    ):
        actual_operations_to_endpoints = []
        for aws_request in cli_runner_result.aws_requests:
            actual_operations_to_endpoints.append(
                (
                    aws_request.operation_name,
                    urlparse.urlparse(aws_request.http_requests[0].url).netloc,
                )
            )
        self.assertEqual(
            actual_operations_to_endpoints, expected_operations_to_endpoints
        )

    def run_command(self, cmdline):
        result = self.cli_runner.run(cmdline)
        self.assertEqual(
            result.rc,
            0,
            f'Expected rc of 0 instead got {result.rc} '
            f'with stderr message: {result.stderr}',
        )
        return result


class BaseCRTTransferClientTest(BaseS3CLIRunnerTest):
    def setUp(self):
        super(BaseCRTTransferClientTest, self).setUp()
        self.crt_client_patch = mock.patch('s3transfer.crt.S3Client')
        self.mock_crt_client = self.crt_client_patch.start()
        self.mock_crt_client.return_value.make_request.side_effect = (
            self.simulate_make_request_side_effect
        )
        self.files = FileCreator()
        self.expected_download_content = b'content'

    def tearDown(self):
        super(BaseCRTTransferClientTest, self).tearDown()
        self.crt_client_patch.stop()
        self.files.remove_all()

    def get_session_stubber(self):
        return IgnoreCRTRequestsSessionStubber()

    def get_config_file_contents(self):
        return (
            '[default]\n'
            's3 =\n'
            '  preferred_transfer_client = crt\n'
            '  max_concurrent_requests = 1\n'
        )

    def simulate_make_request_side_effect(self, *args, **kwargs):
        if kwargs.get('recv_filepath'):
            self.simulate_file_download(kwargs['recv_filepath'])
        elif kwargs.get('on_body'):
            self.simulate_on_body(kwargs['on_body'])
        s3_request = FakeCRTS3Request(
            future=FakeCRTFuture(kwargs.get('on_done'))
        )
        return s3_request

    def simulate_file_download(self, recv_filepath):
        parent_dir = os.path.dirname(recv_filepath)
        if not os.path.isdir(parent_dir):
            os.makedirs(parent_dir)
        with open(recv_filepath, 'wb') as f:
            # The content is arbitrary as most functional tests are just going
            # to assert the file exists since it is the CRT writing the
            # data to the file.
            f.write(self.expected_download_content)

    def simulate_on_body(self, on_body_callback):
        on_body_callback(chunk=self.expected_download_content, offset=0)

    def get_crt_make_request_calls(self):
        return self.mock_crt_client.return_value.make_request.call_args_list

    def assert_crt_client_region(self, expected_region):
        self.assertEqual(
            self.mock_crt_client.call_args[1]['region'], expected_region
        )

    def assert_crt_client_has_no_credential_provider(self):
        self.assertIsNone(
            self.mock_crt_client.call_args[1]['credential_provider']
        )

    def assert_crt_make_request_call(
        self,
        make_request_call,
        expected_type,
        expected_host,
        expected_path,
        expected_http_method=None,
        expected_send_filepath=None,
        expected_recv_startswith=None,
        expected_body_content=None,
    ):
        make_request_kwargs = make_request_call[1]
        self.assertEqual(make_request_kwargs['type'], expected_type)
        self.assertEqual(
            make_request_kwargs['request'].headers.get('host'), expected_host
        )
        self.assertEqual(make_request_kwargs['request'].path, expected_path)
        if expected_http_method:
            self.assertEqual(
                make_request_kwargs['request'].method, expected_http_method
            )
        if expected_send_filepath:
            self.assertEqual(
                make_request_kwargs['send_filepath'], expected_send_filepath
            )
        if expected_recv_startswith:
            # The s3transfer/crt implementation has the CRT client download
            # to a temporary file before moving it to the correct location.
            # This temporary file is just the normal file name with some
            # characters appended to the end. So we are just asserting that
            # the temporary filename being used seems appropriate by seeing if
            # it starts with the final filename as we are not really able to
            # mock out and assert any part of the HTTP layer
            # of the CRT client.
            self.assertTrue(
                make_request_kwargs['recv_filepath'].startswith(
                    expected_recv_startswith
                ),
                (
                    f"{make_request_kwargs['recv_filepath']} does not "
                    f"start with {expected_recv_startswith}"
                ),
            )
        if expected_body_content is not None:
            # Note: The underlying CRT awscrt.io.InputStream does not expose
            # a public read method so we have to reach into the private,
            # underlying stream to determine the content. We should update
            # to use a public interface if a public interface is ever exposed.
            self.assertEqual(
                make_request_kwargs['request'].body_stream._stream.read(),
                expected_body_content,
            )


class FakeCRTS3Request:
    def __init__(self, future):
        self.finished_future = future


class FakeCRTFuture:
    def __init__(self, done_callback=None):
        self._done_callback = done_callback
        self._callback_called = False

    def result(self, timeout=None):
        if self._done_callback and not self._callback_called:
            self._callback_called = True
            self._done_callback(error=None)

    def done(self):
        return True


# The CRT integrations use botocore clients to serialize the request.
# This affects the normal stubber because even though no HTTP requests
# get sent when the client is used to serialize HTTP requests, the stubber
# will still try to provide an HTTP response even though its not needed.
# So this class detects if a call is being made just for the serialization of
# an HTTP request for the CRT client by checking if the request is
# signed as these requests get signed by the CRT client. And if the
# request is intended for the CRT client, it does not send a response back.
class IgnoreCRTRequestsSessionStubber(SessionStubber):
    def _return_queued_http_response(self, request, **kwargs):
        if 'Authorization' not in request.headers:
            return
        response = self._responses.popleft()
        return response.on_http_request_sent(request)



import os

from PyInstaller.utils import hooks

hiddenimports = [
    'docutils',
    'urllib',
    'httplib',
    'html.parser',
    'configparser',
    'xml.etree',
    'pipes',
    'colorama',
    'awscli.handlers',
    # NOTE: This can be removed once this hidden import issue related to
    # setuptools and PyInstaller is resolved:
    # https://github.com/pypa/setuptools/issues/1963
    'pkg_resources.py2_warn',
]

imports_for_legacy_plugins = hooks.collect_submodules(
    'http'
) + hooks.collect_submodules('logging')
hiddenimports += imports_for_legacy_plugins

alias_packages_plugins = hooks.collect_submodules(
    'awscli.botocore'
) + hooks.collect_submodules('awscli.s3transfer')
hiddenimports += alias_packages_plugins


# Completion model files are only used at build time to generate the
# ac.index SQLite database. They are not needed at runtime and can be
# excluded to reduce the size of the PyInstaller distribution.
EXCLUDED_DATA_FILE_BASENAMES = {
    'completions-1.json',
    'completions-1.sdk-extras.json',
}


datas = [
    (src, dest)
    for src, dest in hooks.collect_data_files('awscli')
    if os.path.basename(src) not in EXCLUDED_DATA_FILE_BASENAMES
]


# prompt_toolkit uses its own metadata to determine
# its version. So we need to bundle the package
# metadata to avoid runtime errors.
# https://github.com/aws/aws-cli/issues/9453
datas += hooks.copy_metadata('prompt_toolkit')



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0e
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from botocore.compat import OrderedDict

from awscli.customizations.cloudformation.deployer import Deployer
from awscli.customizations.cloudformation.yamlhelper import (
    yaml_dump,
    yaml_parse,
)
from tests.unit.customizations.cloudformation import BaseYAMLTest


class TestYaml(BaseYAMLTest):
    yaml_with_tags = """
    Resource:
        Key1: !Ref Something
        Key2: !GetAtt Another.Arn
        Key3: !FooBar [!Baz YetAnother, "hello"]
        Key4: !SomeTag {"a": "1"}
        Key5: !GetAtt OneMore.Outputs.Arn
        Key6: !Condition OtherCondition
    """

    parsed_yaml_dict = {
        "Resource": {
            "Key1": {"Ref": "Something"},
            "Key2": {"Fn::GetAtt": ["Another", "Arn"]},
            "Key3": {"Fn::FooBar": [{"Fn::Baz": "YetAnother"}, "hello"]},
            "Key4": {"Fn::SomeTag": {"a": "1"}},
            "Key5": {"Fn::GetAtt": ["OneMore", "Outputs.Arn"]},
            "Key6": {"Condition": "OtherCondition"},
        }
    }

    def test_yaml_with_tags(self):
        output = yaml_parse(self.yaml_with_tags)
        self.assertEqual(self.parsed_yaml_dict, output)

        # Make sure formatter and parser work well with each other
        formatted_str = yaml_dump(output)
        output_again = yaml_parse(formatted_str)
        self.assertEqual(output, output_again)

    def test_yaml_getatt(self):
        # This is an invalid syntax for !GetAtt. But make sure the code does
        # not crash when we encounter this syntax. Let CloudFormation
        # interpret this value at runtime
        yaml_input = """
        Resource:
            Key: !GetAtt ["a", "b"]
        """

        output = {"Resource": {"Key": {"Fn::GetAtt": ["a", "b"]}}}

        actual_output = yaml_parse(yaml_input)
        self.assertEqual(actual_output, output)

    def test_parse_json_with_tabs(self):
        template = '{\n\t"foo": "bar"\n}'
        output = yaml_parse(template)
        self.assertEqual(output, {'foo': 'bar'})

    def test_parse_json_preserve_elements_order(self):
        input_template = """
        {
            "B_Resource": {
                "Key2": {
                    "Name": "name2"
                },
                "Key1": {
                    "Name": "name1"
                }
            },
            "A_Resource": {
                "Key2": {
                    "Name": "name2"
                },
                "Key1": {
                    "Name": "name1"
                }
            }
        }
        """
        expected_dict = OrderedDict(
            [
                (
                    'B_Resource',
                    OrderedDict(
                        [
                            ('Key2', {'Name': 'name2'}),
                            ('Key1', {'Name': 'name1'}),
                        ]
                    ),
                ),
                (
                    'A_Resource',
                    OrderedDict(
                        [
                            ('Key2', {'Name': 'name2'}),
                            ('Key1', {'Name': 'name1'}),
                        ]
                    ),
                ),
            ]
        )
        output_dict = yaml_parse(input_template)
        self.assertEqual(expected_dict, output_dict)

    def test_parse_yaml_preserve_elements_order(self):
        input_template = (
            'B_Resource:\n'
            '  Key2:\n'
            '    Name: name2\n'
            '  Key1:\n'
            '    Name: name1\n'
            'A_Resource:\n'
            '  Key2:\n'
            '    Name: name2\n'
            '  Key1:\n'
            '    Name: name1\n'
        )
        output_dict = yaml_parse(input_template)
        expected_dict = OrderedDict(
            [
                (
                    'B_Resource',
                    OrderedDict(
                        [
                            ('Key2', {'Name': 'name2'}),
                            ('Key1', {'Name': 'name1'}),
                        ]
                    ),
                ),
                (
                    'A_Resource',
                    OrderedDict(
                        [
                            ('Key2', {'Name': 'name2'}),
                            ('Key1', {'Name': 'name1'}),
                        ]
                    ),
                ),
            ]
        )
        self.assertEqual(expected_dict, output_dict)

        output_template = yaml_dump(output_dict)
        self.assertEqual(input_template, output_template)

    def test_yaml_merge_tag(self):
        test_yaml = """
        base: &base
            property: value
        test:
            <<: *base
        """
        output = yaml_parse(test_yaml)
        self.assertTrue(isinstance(output, OrderedDict))
        self.assertEqual(output.get('test').get('property'), 'value')

    def test_unroll_yaml_anchors(self):
        properties = {
            "Foo": "bar",
            "Spam": "eggs",
        }
        template = {
            "Resources": {
                "Resource1": {"Properties": properties},
                "Resource2": {"Properties": properties},
            }
        }

        expected = (
            'Resources:\n'
            '  Resource1:\n'
            '    Properties:\n'
            '      Foo: bar\n'
            '      Spam: eggs\n'
            '  Resource2:\n'
            '    Properties:\n'
            '      Foo: bar\n'
            '      Spam: eggs\n'
        )
        actual = yaml_dump(template)
        self.assertEqual(actual, expected)

    def test_yaml_dump_quotes_boolean_strings(self):
        bools_as_strings = [
            'yes',
            'Yes',
            'YES',
            'no',
            'No',
            'NO',
            'true',
            'True',
            'TRUE',
            'false',
            'False',
            'FALSE',
            'on',
            'On',
            'ON',
            'off',
            'Off',
            'OFF',
        ]
        for bool_as_string in bools_as_strings:
            self.assertEqual(
                yaml_dump(bool_as_string), "'%s'\n" % bool_as_string
            )



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from dataclasses import dataclass

import jmespath
import pytest
from jsonschema import Draft4Validator

from awscli import clidriver
from awscli.botocore.model import ServiceModel
from awscli.botocore.utils import ArgumentGenerator

COMPLETIONS_SCHEMA = {
    "type": "object",
    "properties": {
        "version": {"type": "string"},
        "resources": {
            "type": "object",
            "additionalProperties": {
                "type": "object",
                "properties": {
                    "operation": {"type": "string"},
                    "resourceIdentifier": {
                        "type": "object",
                        "additionalProperties": {"type": "string"},
                        "minProperties": 1,
                    },
                    "inputParameters": {
                        "type": "array",
                        "items": {"type": "string"},
                    },
                },
                "required": ["operation", "resourceIdentifier"],
                "additionalProperties": False,
            },
        },
        "operations": {
            "type": "object",
            "additionalProperties": {
                "type": "object",
                "additionalProperties": {
                    "type": "object",
                    "properties": {
                        "completions": {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "properties": {
                                    "parameters": {
                                        "type": "object",
                                        "additionalProperties": {
                                            "type": "string"
                                        },
                                    },
                                    "resourceName": {"type": "string"},
                                    "resourceIdentifier": {"type": "string"},
                                },
                                "required": [
                                    "resourceName",
                                    "resourceIdentifier",
                                ],
                            },
                        }
                    },
                    "required": ["completions"],
                    "additionalProperties": False,
                },
            },
        },
    },
    "required": ["version", "resources", "operations"],
    "additionalProperties": False,
}


@dataclass
class ServiceTestData:
    service_model: dict
    completions: dict
    service_name: str


def get_models_with_completions():
    session = clidriver.create_clidriver().session
    loader = session.get_component('data_loader')
    services_with_completions = loader.list_available_services('completions-1')
    models = []
    for service_name in services_with_completions:
        service_model = loader.load_service_model(service_name, 'service-2')
        api_version = service_model['metadata']['apiVersion']
        completions = loader.load_service_model(
            service_name, 'completions-1', api_version
        )
        models.append(
            ServiceTestData(service_model, completions, service_name)
        )
    return models


def _get_invalid_completion_operations():
    cases = []
    for test_data in get_models_with_completions():
        known_ops = set(test_data.service_model['operations'])
        for op_name in test_data.completions['operations']:
            if op_name not in known_ops:
                cases.append((test_data.service_name, op_name))
        for resource in test_data.completions.get('resources', {}).values():
            op_name = resource.get('operation')
            if op_name and op_name not in known_ops:
                cases.append((test_data.service_name, op_name))
    return cases


@pytest.mark.validates_models
@pytest.mark.parametrize(
    "service_name, operation_name",
    _get_invalid_completion_operations(),
    ids=lambda val: str(val),
)
def test_completions_operations_exist_in_model(
    service_name, operation_name, record_property
):
    record_property('aws_service', service_name)
    record_property('aws_operation', operation_name)
    pytest.fail(
        f"Completions file for '{service_name}' references operation "
        f"'{operation_name}' which does not exist in the service model. "
        f"The completions-1.json file must be updated to remove or "
        f"update references to this operation."
    )


@pytest.mark.validates_models
@pytest.mark.parametrize(
    "test_data",
    get_models_with_completions(),
    ids=lambda test_data: test_data.service_name,
)
def test_verify_generated_completions_are_valid(test_data):
    completions = test_data.completions
    service_model = test_data.service_model
    _validate_schema(completions)
    # Validate that every operation named in the completions
    # file references a known operation.
    _lint_model_references(completions, service_model)
    _lint_resource_references(completions, service_model)
    _lint_completions(completions, service_model)
    _validate_jmespaths(completions, service_model)


def _get_input_members(service_model, operation_name):
    operation = service_model['operations'][operation_name]
    if 'input' not in operation:
        return {}
    input_shape = operation['input']['shape']
    return service_model['shapes'][input_shape]['members']


def _validate_schema(completions):
    # Ensure completions-1.json file adheres to a JSON schema.
    validator = Draft4Validator(COMPLETIONS_SCHEMA)
    errors = list(e.message for e in validator.iter_errors(completions))
    if errors:
        raise AssertionError('\n'.join(errors))


def _lint_model_references(completions, service_model):
    known_operations = set(service_model['operations'])
    for op_name in completions['operations']:
        assert op_name in known_operations
        # We also want to ensure that all parameters in completions-1.json
        # map to an input member in the service model.
        input_members = _get_input_members(service_model, op_name)
        for param in completions['operations'][op_name]:
            assert param in input_members


def _lint_resource_references(completions, service_model):
    resources = completions['resources']
    for resource_data in resources.values():
        operation_name = resource_data['operation']
        assert operation_name in service_model['operations']

        if 'inputParameters' in resource_data:
            input_members = _get_input_members(service_model, operation_name)
            for param in resource_data['inputParameters']:
                assert param in input_members


def _lint_completions(completions, service_model):
    resources = completions['resources']
    for op_name, completions_data in completions['operations'].items():
        completion_input_members = _get_input_members(service_model, op_name)

        for param_data in completions_data.values():
            for completer_config in param_data['completions']:
                resource_name = completer_config['resourceName']
                resource_id = completer_config['resourceIdentifier']
                assert resource_name in resources
                resource_identifiers = resources[resource_name][
                    'resourceIdentifier'
                ]
                assert resource_id in resource_identifiers

                _validate_completion_parameters(
                    completer_config, resources, completion_input_members
                )


def _validate_completion_parameters(
    completer_config, resources, completion_input_members
):
    resource_name = completer_config['resourceName']
    resource_config = resources[resource_name]
    required_params = resource_config.get('inputParameters', [])
    completion_params = completer_config.get('parameters', {})

    # All required parameters must be provided
    assert set(completion_params.keys()) == set(required_params)

    # Parameter values must be valid in completion operation
    for param_value in completion_params.values():
        assert param_value in completion_input_members


def _validate_jmespaths(completions, service_model):
    service_model_obj = ServiceModel(service_model)
    resources = completions['resources']
    for rconfig in resources.values():
        op_model = service_model_obj.operation_model(rconfig['operation'])
        output_shape = op_model.output_shape
        assert output_shape is not None

        for jmespath_expr in rconfig['resourceIdentifier'].values():
            _validate_jmespath_expression(
                jmespath_expr,
                op_model.output_shape,
            )


def _validate_jmespath_expression(
    jmespath_expr,
    output_shape,
):
    # Test if the JMESPath expression can resolve to a non-empty value
    arg_gen = ArgumentGenerator(use_member_names=True)
    sample_output = arg_gen.generate_skeleton(output_shape)
    search_result = jmespath.search(jmespath_expr, sample_output)
    assert (
        search_result is not None
    ), "Expression is blob or another unsupported type"
    assert search_result, f"Expression is broken: {jmespath_expr}"
    sample_arg = search_result[0]
    assert isinstance(
        sample_arg, str
    ), f"Expression not a string: {jmespath_expr}"



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import json

from awscli.testutils import mock
from tests.unit.customizations.emr import (
    EMRBaseAWSCommandParamsTest as BaseAWSCommandParamsTest,
)

describe_cluster_result_mock_ig = {
    "Cluster": {
        "Status": {
            "Timeline": {
                "ReadyDateTime": 1398376089.0,
                "EndDateTime": 1398376477.0,
                "CreationDateTime": 1398375871.0,
            },
            "State": "TERMINATED",
            "StateChangeReason": {
                "Message": "Terminated by user request",
                "Code": "USER_REQUEST",
            },
        },
        "Ec2InstanceAttributes": {
            "IamInstanceProfile": "EMR_EC2_DefaultRole",
            "Ec2AvailabilityZone": "us-east-1b",
        },
        "InstanceCollectionType": "INSTANCE_GROUP",
        "Name": "ABCD",
        "Tags": [],
        "TerminationProtected": "false",
        "RunningAmiVersion": "2.4.2",
        "Applications": [{"Version": "1.0.3", "Name": "hadoop"}],
        "VisibleToAllUsers": "true",
        "RequestedAmiVersion": "2.4.2",
        "LogUri": "s3n://abc/logs/",
        "AutoTerminate": "false",
        "Id": "j-ABCD",
        "MasterPublicDnsName": "ec2-01-01-1-188.compute-1.amazonaws.com",
    }
}

describe_cluster_result_mock_if = {
    "Cluster": {
        "Status": {
            "Timeline": {
                "ReadyDateTime": 1398376089.0,
                "EndDateTime": 1398376477.0,
                "CreationDateTime": 1398375871.0,
            },
            "State": "TERMINATED",
            "StateChangeReason": {
                "Message": "Terminated by user request",
                "Code": "USER_REQUEST",
            },
        },
        "Ec2InstanceAttributes": {
            "IamInstanceProfile": "EMR_EC2_DefaultRole",
            "Ec2AvailabilityZone": "us-east-1b",
        },
        "InstanceCollectionType": "INSTANCE_FLEET",
        "Name": "ABCD",
        "Tags": [],
        "TerminationProtected": "false",
        "RunningAmiVersion": "2.4.2",
        "Applications": [{"Version": "1.0.3", "Name": "hadoop"}],
        "VisibleToAllUsers": "true",
        "RequestedAmiVersion": "2.4.2",
        "LogUri": "s3n://abc/logs/",
        "AutoTerminate": "false",
        "Id": "j-ABCD",
        "MasterPublicDnsName": "ec2-01-01-1-188.compute-1.amazonaws.com",
    }
}

list_instance_groups_result_mock = {
    "ResponseMetadata": {"RequestId": '12ec323c-50b1-11e4-b653-afa4228d882f'},
    "InstanceGroups": [
        {
            "RequestedInstanceCount": 1,
            "Status": {
                "Timeline": {
                    "ReadyDateTime": 1398376083.0,
                    "EndDateTime": 1398376476.0,
                    "CreationDateTime": 1398375871.0,
                },
                "State": "TERMINATED",
                "StateChangeReason": {
                    "Message": "Job flow terminated",
                    "Code": "CLUSTER_TERMINATED",
                },
            },
            "Name": "Master instance group",
            "InstanceGroupType": "MASTER",
            "InstanceType": "m1.large",
            "CustomAmiId": "ami-deadbeef",
            "Id": "ig-ABCD",
            "Market": "ON_DEMAND",
            "RunningInstanceCount": 0,
        },
        {
            "RequestedInstanceCount": 2,
            "Status": {
                "Timeline": {
                    "ReadyDateTime": 1398376089.0,
                    "EndDateTime": 1398376476.0,
                    "CreationDateTime": 1398375871.0,
                },
                "State": "TERMINATED",
                "StateChangeReason": {
                    "Message": "Job flow terminated",
                    "Code": "CLUSTER_TERMINATED",
                },
            },
            "Name": "Core instance group",
            "InstanceGroupType": "CORE",
            "InstanceType": "m1.large",
            "CustomAmiId": "ami-deadpork",
            "Id": "ig-DEF",
            "Market": "ON_DEMAND",
            "RunningInstanceCount": 0,
        },
    ],
}

list_instance_fleets_result_mock = {
    "ResponseMetadata": {"RequestId": '12ec323c-50b1-11e4-b653-afa4228d882f'},
    "InstanceFleets": [
        {
            "TargetSpotCapacity": 0,
            "TargetOnDemandCapacity": 1,
            "ProvisionedOnDemandCapacity": 0,
            "ProvisionedSpotCapacity": 0,
            "Status": {
                "Timeline": {
                    "ReadyDateTime": 1398376083.0,
                    "EndDateTime": 1398376476.0,
                    "CreationDateTime": 1398375871.0,
                },
                "State": "TERMINATED",
                "StateChangeReason": {
                    "Message": "Job flow terminated",
                    "Code": "CLUSTER_TERMINATED",
                },
            },
            "InstanceTypeSpecifications": [
                {
                    "WeightedCapacity": 1,
                    "BidPrice": "1",
                    "InstanceType": "m3.xlarge",
                    "CustomAmiId": "ami-deadbeef",
                    "BidPriceAsPercentageOfOnDemandPrice": 0.0,
                }
            ],
            "LaunchSpecifications": {
                "SpotSpecification": {
                    "TimeoutDurationMinutes": 77,
                    "TimeoutAction": "TERMINATE_CLUSTER",
                    "AllocationStrategy": "capacity-optimized-prioritized",
                },
                "OnDemandSpecification": {
                    "AllocationStrategy": "lowest-price"
                },
            },
            "ResizeSpecifications": {
                "OnDemandResizeSpecification": {
                    "AllocationStrategy": "lowest-price",
                    "CapacityReservationOptions": {
                        "CapacityReservationPreference": "open",
                        "UsageStrategy": "use-capacity-reservations-first",
                    },
                },
                "SpotResizeSpecification": {
                    "AllocationStrategy": "capacity-optimized"
                },
            },
            "Name": "Master instance group",
            "InstanceFleetType": "MASTER",
            "InstanceType": "m1.large",
            "Id": "if-ABCD",
            "Context": "testContext",
        }
    ],
}

list_bootstrap_actions_result_mock = {
    "ResponseMetadata": {"RequestId": '12ec323c-50b1-11e4-b653-afa4228d882f'},
    "BootstrapActions": [
        {
            "Args": [],
            "Name": "Install HBase",
            "ScriptPath": "s3://elasticmapreduce/bootstrap-actions/setup-hbase",
        }
    ],
}

EXPECTED_RESULT_IG = {
    "Cluster": {
        "Ec2InstanceAttributes": {
            "IamInstanceProfile": "EMR_EC2_DefaultRole",
            "Ec2AvailabilityZone": "us-east-1b",
        },
        "InstanceCollectionType": "INSTANCE_GROUP",
        "Name": "ABCD",
        "TerminationProtected": "false",
        "RunningAmiVersion": "2.4.2",
        "MasterPublicDnsName": "ec2-01-01-1-188.compute-1.amazonaws.com",
        "InstanceGroups": [
            {
                "RequestedInstanceCount": 1,
                "Status": {
                    "Timeline": {
                        "ReadyDateTime": 1398376083.0,
                        "CreationDateTime": 1398375871.0,
                        "EndDateTime": 1398376476.0,
                    },
                    "State": "TERMINATED",
                    "StateChangeReason": {
                        "Message": "Job flow terminated",
                        "Code": "CLUSTER_TERMINATED",
                    },
                },
                "RunningInstanceCount": 0,
                "Name": "Master instance group",
                "InstanceGroupType": "MASTER",
                "InstanceType": "m1.large",
                "CustomAmiId": "ami-deadbeef",
                "Market": "ON_DEMAND",
                "Id": "ig-ABCD",
            },
            {
                "RequestedInstanceCount": 2,
                "Status": {
                    "Timeline": {
                        "ReadyDateTime": 1398376089.0,
                        "CreationDateTime": 1398375871.0,
                        "EndDateTime": 1398376476.0,
                    },
                    "State": "TERMINATED",
                    "StateChangeReason": {
                        "Message": "Job flow terminated",
                        "Code": "CLUSTER_TERMINATED",
                    },
                },
                "RunningInstanceCount": 0,
                "Name": "Core instance group",
                "InstanceGroupType": "CORE",
                "InstanceType": "m1.large",
                "CustomAmiId": "ami-deadpork",
                "Market": "ON_DEMAND",
                "Id": "ig-DEF",
            },
        ],
        "RequestedAmiVersion": "2.4.2",
        "AutoTerminate": "false",
        "LogUri": "s3n://abc/logs/",
        "Status": {
            "Timeline": {
                "ReadyDateTime": 1398376089.0,
                "CreationDateTime": 1398375871.0,
                "EndDateTime": 1398376477.0,
            },
            "State": "TERMINATED",
            "StateChangeReason": {
                "Message": "Terminated by user request",
                "Code": "USER_REQUEST",
            },
        },
        "Tags": [],
        "Applications": [{"Version": "1.0.3", "Name": "hadoop"}],
        "VisibleToAllUsers": "true",
        "BootstrapActions": [
            {
                "Args": [],
                "Name": "Install HBase",
                "ScriptPath": "s3://elasticmapreduce/bootstrap-actions/"
                "setup-hbase",
            }
        ],
        "Id": "j-ABCD",
    }
}

EXPECTED_RESULT_IF = {
    "Cluster": {
        "Ec2InstanceAttributes": {
            "IamInstanceProfile": "EMR_EC2_DefaultRole",
            "Ec2AvailabilityZone": "us-east-1b",
        },
        "InstanceCollectionType": "INSTANCE_FLEET",
        "Name": "ABCD",
        "TerminationProtected": "false",
        "RunningAmiVersion": "2.4.2",
        "MasterPublicDnsName": "ec2-01-01-1-188.compute-1.amazonaws.com",
        "InstanceFleets": [
            {
                "TargetSpotCapacity": 0,
                "TargetOnDemandCapacity": 1,
                "ProvisionedOnDemandCapacity": 0,
                "ProvisionedSpotCapacity": 0,
                "Status": {
                    "Timeline": {
                        "ReadyDateTime": 1398376083.0,
                        "EndDateTime": 1398376476.0,
                        "CreationDateTime": 1398375871.0,
                    },
                    "State": "TERMINATED",
                    "StateChangeReason": {
                        "Message": "Job flow terminated",
                        "Code": "CLUSTER_TERMINATED",
                    },
                },
                "InstanceTypeSpecifications": [
                    {
                        "WeightedCapacity": 1,
                        "BidPrice": "1",
                        "InstanceType": "m3.xlarge",
                        "CustomAmiId": "ami-deadbeef",
                        "BidPriceAsPercentageOfOnDemandPrice": 0.0,
                    }
                ],
                "LaunchSpecifications": {
                    "SpotSpecification": {
                        "TimeoutDurationMinutes": 77,
                        "TimeoutAction": "TERMINATE_CLUSTER",
                        "AllocationStrategy": "capacity-optimized-prioritized",
                    },
                    "OnDemandSpecification": {
                        "AllocationStrategy": "lowest-price"
                    },
                },
                "ResizeSpecifications": {
                    "OnDemandResizeSpecification": {
                        "AllocationStrategy": "lowest-price",
                        "CapacityReservationOptions": {
                            "CapacityReservationPreference": "open",
                            "UsageStrategy": "use-capacity-reservations-first",
                        },
                    },
                    "SpotResizeSpecification": {
                        "AllocationStrategy": "capacity-optimized"
                    },
                },
                "Name": "Master instance group",
                "InstanceFleetType": "MASTER",
                "InstanceType": "m1.large",
                "Id": "if-ABCD",
                "Context": "testContext",
            }
        ],
        "RequestedAmiVersion": "2.4.2",
        "AutoTerminate": "false",
        "LogUri": "s3n://abc/logs/",
        "Status": {
            "Timeline": {
                "ReadyDateTime": 1398376089.0,
                "CreationDateTime": 1398375871.0,
                "EndDateTime": 1398376477.0,
            },
            "State": "TERMINATED",
            "StateChangeReason": {
                "Message": "Terminated by user request",
                "Code": "USER_REQUEST",
            },
        },
        "Tags": [],
        "Applications": [{"Version": "1.0.3", "Name": "hadoop"}],
        "VisibleToAllUsers": "true",
        "BootstrapActions": [
            {
                "Args": [],
                "Name": "Install HBase",
                "ScriptPath": "s3://elasticmapreduce/bootstrap-actions/"
                "setup-hbase",
            }
        ],
        "Id": "j-ABCD",
    }
}


class TestDescribeCluster(BaseAWSCommandParamsTest):
    prefix = 'emr describe-cluster'

    @mock.patch(
        'awscli.customizations.emr.emr.DescribeCluster._construct_result'
    )
    def test_operations_called(self, construct_result_patch):
        construct_result_patch.return_value = dict()

        args = ' --cluster-id j-ABCD'
        cmdline = self.prefix + args

        self.run_cmd(cmdline, expected_rc=0)

        self.assertEqual(len(self.operations_called), 3)
        self.assertEqual(self.operations_called[0][0].name, 'DescribeCluster')
        self.assertEqual(self.operations_called[0][1]['ClusterId'], 'j-ABCD')
        self.assertEqual(
            self.operations_called[1][0].name, 'ListInstanceGroups'
        )
        self.assertEqual(self.operations_called[1][1]['ClusterId'], 'j-ABCD')

        self.assertEqual(
            self.operations_called[2][0].name, 'ListBootstrapActions'
        )
        self.assertEqual(self.operations_called[2][1]['ClusterId'], 'j-ABCD')

    @mock.patch('awscli.customizations.emr.emr.DescribeCluster._call')
    def test_constructed_result_ig(self, call_patch):
        call_patch.side_effect = side_effect_of_call_ig

        args = ' --cluster-id j-ABCD'
        cmdline = self.prefix + args
        result = self.run_cmd(cmdline, expected_rc=0)
        result_json = json.loads(result[0])
        self.assertEqual(result_json, EXPECTED_RESULT_IG)

    @mock.patch('awscli.customizations.emr.emr.DescribeCluster._call')
    def test_constructed_result_if(self, call_patch):
        call_patch.side_effect = side_effect_of_call_if

        args = ' --cluster-id j-ABCD'
        cmdline = self.prefix + args
        result = self.run_cmd(cmdline, expected_rc=0)
        result_json = json.loads(result[0])
        self.assertEqual(result_json, EXPECTED_RESULT_IF)


def side_effect_of_call_ig(*args, **kwargs):
    if args[1] == 'describe_cluster':
        return describe_cluster_result_mock_ig
    elif args[1] == 'list_instance_fleets':
        return list_instance_fleets_result_mock
    elif args[1] == 'list_instance_groups':
        return list_instance_groups_result_mock
    elif args[1] == 'list_bootstrap_actions':
        return list_bootstrap_actions_result_mock


def side_effect_of_call_if(*args, **kwargs):
    if args[1] == 'describe_cluster':
        return describe_cluster_result_mock_if
    elif args[1] == 'list_instance_fleets':
        return list_instance_fleets_result_mock
    elif args[1] == 'list_instance_groups':
        return list_instance_groups_result_mock
    elif args[1] == 'list_bootstrap_actions':
        return list_bootstrap_actions_result_mock


if __name__ == "__main__":
    unittest.main()



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from argparse import Namespace
from decimal import Decimal

import jmespath
import pytest

from awscli.customizations.dynamodb.formatter import DynamoYAMLDumper
from awscli.customizations.dynamodb.types import Binary
from awscli.formatter import YAMLFormatter
from awscli.testutils import capture_output


@pytest.mark.parametrize(
    'case',
    [
        {
            'given': {'mykey': Decimal('1')},
            'expected': '1\n',
            'query': 'mykey',
        },
        {
            'given': {'mykey': Decimal('1.1')},
            'expected': '1.1\n',
            'query': 'mykey',
        },
        {
            'given': {'mykey': Decimal('1')},
            'expected': 'mykey: 1\n',
        },
        {
            'given': {'mykey': Decimal('1.1')},
            'expected': 'mykey: 1.1\n',
        },
        {
            'given': {'mykey': Binary(b'\xae\x85\x8b\xd7')},
            'expected': 'mykey: !!binary "roWL1w=="\n',
        },
        {
            'given': {'mykey': Binary(b'\xae\x85\x8b\xd7')},
            'expected': '!!binary "roWL1w=="\n',
            'query': 'mykey',
        },
    ],
)
def test_yaml_formatter_with_dynamo_dumper(case):
    compiled_query = None
    if case.get('query') is not None:
        compiled_query = jmespath.compile(case['query'])
    formatter = YAMLFormatter(
        Namespace(query=compiled_query), DynamoYAMLDumper()
    )
    with capture_output() as output:
        formatter('fake-command-name', case['given'])
    assert output.stdout.getvalue() == case['expected']



# Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.autocomplete.db import DatabaseConnection
from awscli.testutils import mock, unittest


class TestDatabaseConnection(unittest.TestCase):
    @mock.patch('awscli.compat.sqlite3.connect')
    def test_does_try_to_enable_wal(self, mock_connect):
        conn = DatabaseConnection(':memory:')
        mock_connection = mock_connect.return_value
        conn.execute('my SQL query')
        mock_connection.execute.assert_any_call('PRAGMA journal_mode=OFF', {})



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.autocomplete import generator
from awscli.autocomplete.local import indexer
from awscli.clidriver import CLIDriver
from awscli.testutils import mock, unittest


class TestGenerateCompletionIndex(unittest.TestCase):
    def test_use_high_level_generator_for_index_creation(self):
        model_index = mock.Mock(spec=indexer.ModelIndexer)
        clidriver = mock.Mock(spec=CLIDriver)
        index = generator.IndexGenerator([model_index])
        index.generate_index(clidriver)
        model_index.generate_index.assert_called_with(clidriver)



# Copyright 2021 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import errno
import json

from awscli.testutils import (
    BaseAWSCommandParamsTest,
    BaseAWSHelpOutputTest,
    mock,
)


class TestExecuteCommand(BaseAWSCommandParamsTest):
    @mock.patch('awscli.customizations.ecs.executecommand.check_call')
    def test_execute_command_success(self, mock_check_call):
        cmdline = (
            'ecs execute-command --cluster someCluster '
            '--task someTaskId '
            '--interactive --command ls '
            '--region us-west-2'
        )
        mock_check_call.return_value = 0
        self.parsed_responses = [
            {
                "containerName": "someContainerName",
                "containerArn": "someContainerArn",
                "taskArn": "someTaskArn",
                "session": {
                    "sessionId": "session-id",
                    "tokenValue": "token-value",
                    "streamUrl": "stream-url",
                },
                "clusterArn": "someCluster",
                "interactive": "true",
            },
            {
                "failures": [],
                "tasks": [
                    {
                        "clusterArn": "ecs/someCLuster",
                        "desiredStatus": "RUNNING",
                        "createdAt": "1611619514.46",
                        "taskArn": "someTaskArn",
                        "containers": [
                            {
                                "containerArn": "ecs/someContainerArn",
                                "taskArn": "ecs/someTaskArn",
                                "name": "someContainerName",
                                "managedAgents": [
                                    {
                                        "reason": "Execute Command Agent started",
                                        "lastStatus": "RUNNING",
                                        "lastStartedAt": "1611619528.272",
                                        "name": "ExecuteCommandAgent",
                                    }
                                ],
                                "runtimeId": "someRuntimeId",
                            },
                            {
                                "containerArn": "ecs/dummyContainerArn",
                                "taskArn": "ecs/someTaskArn",
                                "name": "dummyContainerName",
                                "managedAgents": [
                                    {
                                        "reason": "Execute Command Agent started",
                                        "lastStatus": "RUNNING",
                                        "lastStartedAt": "1611619528.272",
                                        "name": "ExecuteCommandAgent",
                                    }
                                ],
                                "runtimeId": "dummyRuntimeId",
                            },
                        ],
                        "lastStatus": "RUNNING",
                        "enableExecuteCommand": "true",
                    }
                ],
            },
        ]
        self.run_cmd(cmdline, expected_rc=0)
        self.assertEqual(self.operations_called[0][0].name, 'ExecuteCommand')
        actual_response = json.loads(mock_check_call.call_args[0][0][1])
        self.assertEqual(
            {
                "sessionId": "session-id",
                "tokenValue": "token-value",
                "streamUrl": "stream-url",
            },
            actual_response,
        )

    @mock.patch('awscli.customizations.ecs.executecommand.check_call')
    def test_execute_command_fails(self, mock_check_call):
        cmdline = (
            'ecs execute-command --cluster someCluster '
            '--task someTaskId '
            '--interactive --command ls '
            '--region us-west-2'
        )
        mock_check_call.side_effect = OSError(errno.ENOENT, 'some error')
        self.run_cmd(cmdline, expected_rc=255)


class TestHelpOutput(BaseAWSHelpOutputTest):
    def test_execute_command_output(self):
        self.driver.main(['ecs', 'execute-command', 'help'])
        self.assert_contains('Output\n======\n\nNone')



# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from awscli.testutils import BaseAWSCommandParamsTest


class TestPutEventsCommand(BaseAWSCommandParamsTest):
    def run_put_events(self, cmd):
        response, _, _ = self.run_cmd(cmd)
        return response

    def test_put_events_with_endpoint_id(self):
        """Verify SigV4a signer has been loaded for put-events with
        an endpoint-id provided.
        """
        self.patch_make_request()
        cmd = [
            "events",
            "put-events",
            "--entries",
            "[{}]",
            "--endpoint-id",
            "123abcdefg.abc",
        ]
        response = self.run_put_events(cmd)
        self.assertEqual(
            self.last_request_dict["url"],
            "https://123abcdefg.abc.endpoint.events.amazonaws.com/",
        )
        self.assertEqual(self.last_request_dict["context"]["auth_type"], "v4a")



# Copyright 2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os
import unittest

from awscli.customizations.s3.fileformat import FileFormat


class FileFormatTest(unittest.TestCase):
    def setUp(self):
        self.file_format = FileFormat()

    def test_op_dir(self):
        """
        Format a paths for directory operation.  There are slashes at the
        end of the paths.
        """
        src = '.' + os.sep
        dest = 's3://kyknapp/golfVid/'
        parameters = {'dir_op': True}
        files = self.file_format.format(src, dest, parameters)

        ref_files = {
            'src': {'path': os.path.abspath(src) + os.sep, 'type': 'local'},
            'dest': {'path': 'kyknapp/golfVid/', 'type': 's3'},
            'dir_op': True,
            'use_src_name': True,
        }
        self.assertEqual(files, ref_files)

    def test_op_dir_noslash(self):
        """
        Format a paths for directory operation.  There are no slashes at the
        end of the paths.
        """
        src = '.'
        dest = 's3://kyknapp/golfVid'
        parameters = {'dir_op': True}
        files = self.file_format.format(src, dest, parameters)

        ref_files = {
            'src': {'path': os.path.abspath(src) + os.sep, 'type': 'local'},
            'dest': {'path': 'kyknapp/golfVid/', 'type': 's3'},
            'dir_op': True,
            'use_src_name': True,
        }
        self.assertEqual(files, ref_files)

    def test_local_use_src_name(self):
        """
        No directory operation. S3 source name given. Existing local
        destination directory given.
        """
        src = 's3://kyknapp/golfVid/hello.txt'
        dest = '.'
        parameters = {'dir_op': False}
        files = self.file_format.format(src, dest, parameters)

        ref_files = {
            'src': {'path': 'kyknapp/golfVid/hello.txt', 'type': 's3'},
            'dest': {'path': os.path.abspath(dest) + os.sep, 'type': 'local'},
            'dir_op': False,
            'use_src_name': True,
        }
        self.assertEqual(files, ref_files)

    def test_local_noexist_file(self):
        """
        No directory operation. S3 source name given. Nonexisting local
        destination directory given.
        """
        src = 's3://kyknapp/golfVid/hello.txt'
        dest = 'someFile' + os.sep
        parameters = {'dir_op': False}
        files = self.file_format.format(src, dest, parameters)

        ref_files = {
            'src': {'path': 'kyknapp/golfVid/hello.txt', 'type': 's3'},
            'dest': {'path': os.path.abspath(dest) + os.sep, 'type': 'local'},
            'dir_op': False,
            'use_src_name': True,
        }
        self.assertEqual(files, ref_files)

    def test_local_keep_dest_name(self):
        """
        No directory operation. S3 source name given. Local
        destination filename given.
        """
        src = 's3://kyknapp/golfVid/hello.txt'
        dest = 'hello.txt'
        parameters = {'dir_op': False}
        files = self.file_format.format(src, dest, parameters)

        ref_files = {
            'src': {'path': 'kyknapp/golfVid/hello.txt', 'type': 's3'},
            'dest': {'path': os.path.abspath(dest), 'type': 'local'},
            'dir_op': False,
            'use_src_name': False,
        }
        self.assertEqual(files, ref_files)

    def test_s3_use_src_name(self):
        """
        No directory operation. Local source name given. S3
        common prefix given.
        """
        src = 'fileformat_test.py'
        dest = 's3://kyknapp/golfVid/'
        parameters = {'dir_op': False}
        files = self.file_format.format(src, dest, parameters)

        ref_files = {
            'src': {'path': os.path.abspath(src), 'type': 'local'},
            'dest': {'path': 'kyknapp/golfVid/', 'type': 's3'},
            'dir_op': False,
            'use_src_name': True,
        }
        self.assertEqual(files, ref_files)

    def test_s3_keep_dest_name(self):
        """
        No directory operation. Local source name given. S3
        full key given.
        """
        src = 'fileformat_test.py'
        dest = 's3://kyknapp/golfVid/file.py'
        parameters = {'dir_op': False}
        files = self.file_format.format(src, dest, parameters)

        ref_files = {
            'src': {'path': os.path.abspath(src), 'type': 'local'},
            'dest': {'path': 'kyknapp/golfVid/file.py', 'type': 's3'},
            'dir_op': False,
            'use_src_name': False,
        }
        self.assertEqual(files, ref_files)


if __name__ == "__main__":
    unittest.main()



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/

from unittest.mock import Mock, patch

import pytest
from botocore.exceptions import ClientError

from awscli.customizations.ecs.expressgateway.display_strategy import (
    InteractiveDisplayStrategy
)
from awscli.customizations.ecs.exceptions import MonitoringError
from awscli.customizations.ecs.monitorexpressgatewayservice import (
    ECSExpressGatewayServiceWatcher,
    ECSMonitorExpressGatewayService,
)
from awscli.customizations.ecs.prompt_toolkit_display import Display


class TestECSMonitorExpressGatewayServiceCommand:
    """Test the command class through public interface"""

    @patch('sys.stdout.isatty')
    def test_monitoring_error_handled_gracefully(self, mock_isatty, capsys):
        """Test MonitoringError is caught and printed"""
        # Mock TTY for interactive mode
        mock_isatty.return_value = True

        mock_session = Mock()
        mock_client = Mock()
        mock_session.create_client.return_value = mock_client

        mock_watcher_class = Mock()
        mock_watcher = Mock()
        mock_watcher.exec.side_effect = MonitoringError("Test error")
        mock_watcher_class.return_value = mock_watcher

        command = ECSMonitorExpressGatewayService(
            mock_session, watcher_class=mock_watcher_class
        )

        parsed_args = Mock(
            service_arn="test-arn", resource_view="RESOURCE", timeout=30
        )
        parsed_globals = Mock(
            region="us-west-2", endpoint_url=None, verify_ssl=True
        )

        command._run_main(parsed_args, parsed_globals)

        captured = capsys.readouterr()
        assert "Error monitoring service: Test error" in captured.err

    @patch('sys.stdout.isatty')
    def test_non_monitoring_error_bubbles_up(self, mock_isatty):
        """Test non-MonitoringError exceptions are not caught"""
        # Mock TTY for interactive mode
        mock_isatty.return_value = True

        mock_session = Mock()
        command = ECSMonitorExpressGatewayService(mock_session)

        mock_session.create_client.side_effect = ValueError("Unexpected error")

        parsed_args = Mock(
            service_arn="test-arn", resource_view="RESOURCE", timeout=30
        )
        parsed_globals = Mock(
            region="us-west-2", endpoint_url=None, verify_ssl=True
        )

        with pytest.raises(ValueError):
            command._run_main(parsed_args, parsed_globals)

    @patch('sys.stdout.isatty')
    def test_interactive_mode_requires_tty(self, mock_isatty, capsys):
        """Test interactive mode fails without TTY"""
        # Not in TTY
        mock_isatty.return_value = False

        mock_session = Mock()
        mock_client = Mock()
        mock_session.create_client.return_value = mock_client

        command = ECSMonitorExpressGatewayService(mock_session)

        parsed_args = Mock(
            service_arn="test-arn",
            resource_view="RESOURCE",
            timeout=30,
            mode='INTERACTIVE',
        )
        parsed_globals = Mock(
            region="us-west-2",
            endpoint_url=None,
            verify_ssl=True,
            color='auto',
        )

        result = command._run_main(parsed_args, parsed_globals)

        captured = capsys.readouterr()
        assert "Interactive mode requires a TTY" in captured.err
        assert "aws: [ERROR]:" in captured.err
        assert result == 1

    @patch('sys.stdout.isatty')
    def test_text_only_mode_without_tty(self, mock_isatty, capsys):
        """Test command uses text-only mode when not in TTY"""
        # Not in TTY
        mock_isatty.return_value = False

        mock_session = Mock()
        mock_client = Mock()
        mock_session.create_client.return_value = mock_client

        mock_watcher_class = Mock()
        mock_watcher = Mock()
        mock_watcher_class.return_value = mock_watcher

        command = ECSMonitorExpressGatewayService(
            mock_session, watcher_class=mock_watcher_class
        )

        parsed_args = Mock(
            service_arn="test-arn",
            resource_view="RESOURCE",
            timeout=30,
            mode=None,
        )
        parsed_globals = Mock(
            region="us-west-2", endpoint_url=None, verify_ssl=True
        )

        command._run_main(parsed_args, parsed_globals)


@pytest.fixture
def service_arn():
    """Fixture that provides a test service ARN."""
    return "arn:aws:ecs:us-west-2:123456789012:service/my-cluster/my-service"


class TestECSExpressGatewayServiceWatcher:
    """Test the watcher class through public interface"""

    def test_init_creates_collector_with_correct_parameters(self):
        """Test watcher creates collector with correct client, service_arn, resource_view, use_color"""
        mock_client = Mock()
        service_arn = "arn:aws:ecs:us-west-2:123456789012:service/test-service"

        watcher = ECSExpressGatewayServiceWatcher(
            mock_client,
            service_arn,
            resource_view="DEPLOYMENT",
            display_mode="TEXT-ONLY",
            use_color=False,
        )

        # Verify collector was created with correct parameters
        assert watcher.collector is not None
        assert watcher.collector._client == mock_client
        assert watcher.collector.service_arn == service_arn
        assert watcher.collector.mode == "DEPLOYMENT"
        assert watcher.collector.use_color is False

    def test_init_uses_injected_collector(self):
        """Test watcher uses injected collector instead of creating one"""
        mock_collector = Mock()
        mock_display_strategy = Mock()

        watcher = ECSExpressGatewayServiceWatcher(
            Mock(),
            "arn:aws:ecs:us-west-2:123456789012:service/test-service",
            "RESOURCE",
            "INTERACTIVE",
            collector=mock_collector,
            display_strategy=mock_display_strategy,
        )

        assert watcher.collector == mock_collector

    def test_exec_calls_display_strategy_with_correct_parameters(
        self, ptk_app_session
    ):
        """Test exec() calls display strategy with collector, start_time, and timeout"""
        mock_collector = Mock()
        mock_display_strategy = Mock()

        watcher = ECSExpressGatewayServiceWatcher(
            Mock(),
            "arn:aws:ecs:us-west-2:123456789012:service/test-service",
            "RESOURCE",
            "INTERACTIVE",
            timeout_minutes=15,
            display_strategy=mock_display_strategy,
            collector=mock_collector,
        )

        watcher.exec()

        # Verify display strategy was called once
        mock_display_strategy.execute_monitoring.assert_called_once()

        # Verify correct parameters were passed
        call_args = mock_display_strategy.execute_monitoring.call_args
        assert call_args.kwargs['collector'] == mock_collector
        assert call_args.kwargs['start_time'] == watcher.start_time
        assert call_args.kwargs['timeout_minutes'] == 15

    def test_exec_propagates_exceptions_from_display_strategy(
        self, ptk_app_session
    ):
        """Test exec() propagates exceptions from display strategy"""
        mock_display_strategy = Mock()
        mock_display_strategy.execute_monitoring.side_effect = ClientError(
            error_response={
                'Error': {
                    'Code': 'ServiceNotFoundException',
                    'Message': 'Service not found',
                }
            },
            operation_name='DescribeExpressGatewayService',
        )

        watcher = ECSExpressGatewayServiceWatcher(
            Mock(),
            "arn:aws:ecs:us-west-2:123456789012:service/test-service",
            "RESOURCE",
            "INTERACTIVE",
            display_strategy=mock_display_strategy,
            collector=Mock(),
        )

        with pytest.raises(ClientError) as exc_info:
            watcher.exec()

        assert (
            exc_info.value.response['Error']['Code']
            == 'ServiceNotFoundException'
        )
        assert (
            exc_info.value.response['Error']['Message'] == 'Service not found'
        )


class TestMonitoringError:
    """Test MonitoringError exception class"""

    def test_monitoring_error_creation(self):
        """Test MonitoringError can be created with message"""
        error = MonitoringError("Test error")
        assert str(error) == "Test error"
        assert error.message == "Test error"


class TestColorSupport:
    """Test color support functionality"""

    @pytest.fixture
    def display(self, ptk_app_session):
        yield Display()

    def test_should_use_color_on(self):
        """Test _should_use_color returns True when color is 'on'"""
        command = ECSMonitorExpressGatewayService(Mock())
        parsed_globals = Mock()
        parsed_globals.color = 'on'

        assert command._should_use_color(parsed_globals) is True

    def test_should_use_color_off(self):
        """Test _should_use_color returns False when color is 'off'"""
        command = ECSMonitorExpressGatewayService(Mock())
        parsed_globals = Mock()
        parsed_globals.color = 'off'

        assert command._should_use_color(parsed_globals) is False

    @patch('sys.stdout.isatty')
    def test_should_use_color_auto_tty(self, mock_isatty):
        """Test _should_use_color returns True for 'auto' when stdout is TTY"""
        mock_isatty.return_value = True
        command = ECSMonitorExpressGatewayService(Mock())
        parsed_globals = Mock()
        parsed_globals.color = 'auto'

        assert command._should_use_color(parsed_globals) is True

    @patch('sys.stdout.isatty')
    def test_should_use_color_auto_no_tty(self, mock_isatty):
        """Test _should_use_color returns False for 'auto' when stdout is not TTY"""
        mock_isatty.return_value = False
        command = ECSMonitorExpressGatewayService(Mock())
        parsed_globals = Mock()
        parsed_globals.color = 'auto'

        assert command._should_use_color(parsed_globals) is False

    @pytest.mark.parametrize("use_color", [True, False])
    def test_watcher_accepts_use_color_parameter(self, display, use_color):
        """Test ECSExpressGatewayServiceWatcher accepts use_color parameter"""
        mock_client = Mock()

        watcher = ECSExpressGatewayServiceWatcher(
            mock_client,
            "arn:aws:ecs:us-east-1:123456789012:service/test-service",
            "ALL",
            "INTERACTIVE",
            use_color=use_color,
            display_strategy=InteractiveDisplayStrategy(display, use_color),
        )
        assert watcher.collector.use_color is use_color

    def test_invalid_display_mode_raises_error(self):
        """Test that invalid display mode raises ValueError"""
        mock_client = Mock()

        with pytest.raises(ValueError) as exc_info:
            ECSExpressGatewayServiceWatcher(
                mock_client,
                "arn:aws:ecs:us-east-1:123456789012:service/test-service",
                "RESOURCE",
                "INVALID_MODE",
            )
        assert "Invalid display mode: INVALID_MODE" in str(exc_info.value)



#!/usr/bin/env python
# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os

from awscrt.s3 import S3RequestType

from awscli.compat import BytesIO
from awscli.testutils import mock, skip_if_case_sensitive
from tests import requires_crt
from tests.functional.s3 import (
    BaseCRTTransferClientTest,
    BaseS3TransferCommandTest,
)
from tests.functional.s3.test_sync_command import TestSyncCaseConflict


class TestMvCommand(BaseS3TransferCommandTest):
    prefix = 's3 mv '

    def test_cant_mv_object_onto_itself(self):
        cmdline = '%s s3://bucket/key s3://bucket/key' % self.prefix
        stderr = self.run_cmd(cmdline, expected_rc=252)[1]
        self.assertIn('Cannot mv a file onto itself', stderr)

    def test_cant_mv_object_with_implied_name(self):
        # The "key" key name is implied in the dst argument.
        cmdline = '%s s3://bucket/key s3://bucket/' % self.prefix
        stderr = self.run_cmd(cmdline, expected_rc=252)[1]
        self.assertIn('Cannot mv a file onto itself', stderr)

    def test_dryrun_move(self):
        self.parsed_responses = [self.head_object_response()]
        cmdline = (
            f'{self.prefix} s3://bucket/key.txt s3://bucket/key2.txt --dryrun'
        )
        stdout, _, _ = self.run_cmd(cmdline, expected_rc=0)
        self.assert_operations_called(
            [
                (
                    'HeadObject',
                    {
                        'Bucket': 'bucket',
                        'Key': 'key.txt',
                        'ChecksumMode': 'ENABLED',
                    },
                )
            ]
        )
        self.assertIn(
            '(dryrun) move: s3://bucket/key.txt to s3://bucket/key2.txt',
            stdout,
        )

    def test_website_redirect_ignore_paramfile(self):
        full_path = self.files.create_file('foo.txt', 'mycontent')
        cmdline = '%s %s s3://bucket/key.txt --website-redirect %s' % (
            self.prefix,
            full_path,
            'http://someserver',
        )
        self.parsed_responses = [
            {'ETag': '"c8afdb36c52cf4727836669019e69222"'}
        ]
        self.run_cmd(cmdline, expected_rc=0)
        self.assertEqual(self.operations_called[0][0].name, 'PutObject')
        # Make sure that the specified web address is used as opposed to the
        # contents of the web address.
        self.assertEqual(
            self.operations_called[0][1]['WebsiteRedirectLocation'],
            'http://someserver',
        )

    def test_metadata_directive_copy(self):
        self.parsed_responses = [
            {
                "ContentLength": "100",
                "LastModified": "00:00:00Z",
                'ETag': '"foo"',
            },
            {'ETag': '"foo-1"'},
            {'ETag': '"foo-2"'},
        ]
        cmdline = (
            '%s s3://bucket/key.txt s3://bucket/key2.txt'
            ' --metadata-directive REPLACE' % self.prefix
        )
        self.run_cmd(cmdline, expected_rc=0)
        self.assertEqual(
            len(self.operations_called), 3, self.operations_called
        )
        self.assertEqual(self.operations_called[0][0].name, 'HeadObject')
        self.assertEqual(self.operations_called[1][0].name, 'CopyObject')
        self.assertEqual(self.operations_called[2][0].name, 'DeleteObject')
        self.assertEqual(
            self.operations_called[1][1]['MetadataDirective'], 'REPLACE'
        )

    def test_no_metadata_directive_for_non_copy(self):
        full_path = self.files.create_file('foo.txt', 'mycontent')
        cmdline = '%s %s s3://bucket --metadata-directive REPLACE' % (
            self.prefix,
            full_path,
        )
        self.parsed_responses = [
            {'ETag': '"c8afdb36c52cf4727836669019e69222"'}
        ]
        self.run_cmd(cmdline, expected_rc=0)
        self.assertEqual(
            len(self.operations_called), 1, self.operations_called
        )
        self.assertEqual(self.operations_called[0][0].name, 'PutObject')
        self.assertNotIn('MetadataDirective', self.operations_called[0][1])

    def test_download_move_with_request_payer(self):
        cmdline = '%s s3://mybucket/mykey %s --request-payer' % (
            self.prefix,
            self.files.rootdir,
        )

        self.parsed_responses = [
            # Response for HeadObject
            {
                "ContentLength": 100,
                "LastModified": "00:00:00Z",
                "ETag": '"foo-1"',
            },
            # Response for GetObject
            {'ETag': '"foo-1"', 'Body': BytesIO(b'foo')},
            # Response for DeleteObject
            {},
        ]

        self.run_cmd(cmdline, expected_rc=0)
        self.assert_operations_called(
            [
                (
                    'HeadObject',
                    {
                        'Bucket': 'mybucket',
                        'Key': 'mykey',
                        'ChecksumMode': 'ENABLED',
                        'RequestPayer': 'requester',
                    },
                ),
                (
                    'GetObject',
                    {
                        'Bucket': 'mybucket',
                        'Key': 'mykey',
                        'RequestPayer': 'requester',
                    },
                ),
                (
                    'DeleteObject',
                    {
                        'Bucket': 'mybucket',
                        'Key': 'mykey',
                        'RequestPayer': 'requester',
                    },
                ),
            ]
        )

    def test_copy_move_with_request_payer(self):
        cmdline = self.prefix
        cmdline += 's3://sourcebucket/sourcekey s3://mybucket/mykey'
        cmdline += ' --request-payer'

        self.parsed_responses = [
            self.head_object_response(),
            self.copy_object_response(),
            self.delete_object_response(),
        ]
        self.run_cmd(cmdline, expected_rc=0)
        self.assert_operations_called(
            [
                self.head_object_request(
                    'sourcebucket',
                    'sourcekey',
                    RequestPayer='requester',
                    ChecksumMode='ENABLED',
                ),
                self.copy_object_request(
                    'sourcebucket',
                    'sourcekey',
                    'mybucket',
                    'mykey',
                    RequestPayer='requester',
                ),
                self.delete_object_request(
                    'sourcebucket', 'sourcekey', RequestPayer='requester'
                ),
            ]
        )

    def test_with_copy_props(self):
        cmdline = self.prefix
        cmdline += 's3://sourcebucket/sourcekey s3://bucket/key'
        cmdline += ' --copy-props default'

        upload_id = 'upload_id'
        large_tag_set = {'tag-key': 'val' * 3000}
        metadata = {'tag-key': 'tag-value'}
        self.parsed_responses = [
            self.head_object_response(
                Metadata=metadata, ContentLength=8 * 1024**2
            ),
            self.get_object_tagging_response(large_tag_set),
            self.create_mpu_response(upload_id),
            self.upload_part_copy_response(),
            self.complete_mpu_response(),
            self.put_object_tagging_response(),
            self.delete_object_response(),
        ]
        self.run_cmd(cmdline, expected_rc=0)
        self.assert_operations_called(
            [
                self.head_object_request(
                    'sourcebucket', 'sourcekey', ChecksumMode='ENABLED'
                ),
                self.get_object_tagging_request('sourcebucket', 'sourcekey'),
                self.create_mpu_request('bucket', 'key', Metadata=metadata),
                self.upload_part_copy_request(
                    'sourcebucket',
                    'sourcekey',
                    'bucket',
                    'key',
                    upload_id,
                    CopySourceRange=mock.ANY,
                    PartNumber=1,
                    CopySourceIfMatch='"foo-1"',
                ),
                self.complete_mpu_request('bucket', 'key', upload_id, 1),
                self.put_object_tagging_request(
                    'bucket', 'key', large_tag_set
                ),
                self.delete_object_request('sourcebucket', 'sourcekey'),
            ]
        )

    def test_mv_does_not_delete_source_on_failed_put_tagging(self):
        cmdline = self.prefix
        cmdline += 's3://sourcebucket/sourcekey s3://bucket/key'
        cmdline += ' --copy-props default'

        upload_id = 'upload_id'
        large_tag_set = {'tag-key': 'val' * 3000}
        metadata = {'tag-key': 'tag-value'}
        self.parsed_responses = [
            self.head_object_response(
                Metadata=metadata, ContentLength=8 * 1024**2
            ),
            self.get_object_tagging_response(large_tag_set),
            self.create_mpu_response(upload_id),
            self.upload_part_copy_response(),
            self.complete_mpu_response(),
            self.access_denied_error_response(),  # PutObjectTagging error
            self.delete_object_response(),
        ]
        self.set_http_status_codes(
            [
                200,  # HeadObject
                200,  # GetObjectTagging
                200,  # CreateMultipartUpload
                200,  # UploadPartCopy
                200,  # CompleteMultipartUpload
                403,  # PutObjectTagging
                200,  # DeleteObject
            ]
        )
        self.run_cmd(cmdline, expected_rc=1)
        self.assert_operations_called(
            [
                self.head_object_request(
                    'sourcebucket', 'sourcekey', ChecksumMode='ENABLED'
                ),
                self.get_object_tagging_request('sourcebucket', 'sourcekey'),
                self.create_mpu_request('bucket', 'key', Metadata=metadata),
                self.upload_part_copy_request(
                    'sourcebucket',
                    'sourcekey',
                    'bucket',
                    'key',
                    upload_id,
                    CopySourceRange=mock.ANY,
                    PartNumber=1,
                    CopySourceIfMatch='"foo-1"',
                ),
                self.complete_mpu_request('bucket', 'key', upload_id, 1),
                self.put_object_tagging_request(
                    'bucket', 'key', large_tag_set
                ),
                self.delete_object_request('bucket', 'key'),
            ]
        )

    def test_upload_with_checksum_algorithm_crc32(self):
        full_path = self.files.create_file('foo.txt', 'contents')
        cmdline = f'{self.prefix} {full_path} s3://bucket/key.txt --checksum-algorithm CRC32'
        self.run_cmd(cmdline, expected_rc=0)
        self.assertEqual(self.operations_called[0][0].name, 'PutObject')
        self.assertEqual(
            self.operations_called[0][1]['ChecksumAlgorithm'], 'CRC32'
        )

    def test_download_with_checksum_mode_crc32(self):
        self.parsed_responses = [
            self.head_object_response(),
            # Mocked GetObject response with a checksum algorithm specified
            {
                'ETag': 'foo-1',
                'ChecksumCRC32': 'checksum',
                'Body': BytesIO(b'foo'),
            },
            self.delete_object_response(),
        ]
        cmdline = f'{self.prefix} s3://bucket/foo {self.files.rootdir} --checksum-mode ENABLED'
        self.run_cmd(cmdline, expected_rc=0)
        self.assertEqual(self.operations_called[1][0].name, 'GetObject')
        self.assertEqual(
            self.operations_called[1][1]['ChecksumMode'], 'ENABLED'
        )

    def test_mv_no_overwrite_flag_when_object_not_exists_on_target(self):
        full_path = self.files.create_file('foo.txt', 'contents')
        cmdline = f'{self.prefix} {full_path} s3://bucket --no-overwrite'
        self.run_cmd(cmdline, expected_rc=0)
        # Verify putObject was called
        self.assertEqual(len(self.operations_called), 1)
        self.assertEqual(self.operations_called[0][0].name, 'PutObject')
        # Verify the IfNoneMatch condition was set in the request
        self.assertEqual(self.operations_called[0][1]['IfNoneMatch'], '*')
        # Verify source file was deleted (move operation)
        self.assertFalse(os.path.exists(full_path))

    def test_mv_no_overwrite_flag_when_object_exists_on_target(self):
        full_path = self.files.create_file('foo.txt', 'mycontent')
        cmdline = (
            f'{self.prefix} {full_path} s3://bucket/foo.txt --no-overwrite'
        )
        # Set up the response to simulate a PreconditionFailed error
        self.http_response.status_code = 412
        self.parsed_responses = [
            self.precondition_failed_error_response(),
        ]
        self.run_cmd(cmdline, expected_rc=0)
        # Verify PutObject was attempted with IfNoneMatch
        self.assertEqual(len(self.operations_called), 1)
        self.assertEqual(self.operations_called[0][0].name, 'PutObject')
        self.assertEqual(self.operations_called[0][1]['IfNoneMatch'], '*')
        # Verify source file was not deleted
        self.assertTrue(os.path.exists(full_path))

    def test_mv_no_overwrite_flag_multipart_upload_when_object_not_exists_on_target(
        self,
    ):
        # Create a large file that will trigger multipart upload
        full_path = self.files.create_file('foo.txt', 'a' * 10 * (1024**2))
        cmdline = f'{self.prefix} {full_path} s3://bucket --no-overwrite'
        # Set up responses for multipart upload
        self.parsed_responses = [
            {'UploadId': 'foo'},  # CreateMultipartUpload response
            {'ETag': '"foo-1"'},  # UploadPart response
            {'ETag': '"foo-2"'},  # UploadPart response
            {},  # CompleteMultipartUpload response
        ]
        self.run_cmd(cmdline, expected_rc=0)
        # Verify all multipart operations were called
        self.assertEqual(len(self.operations_called), 4)
        self.assertEqual(
            self.operations_called[0][0].name, 'CreateMultipartUpload'
        )
        self.assertEqual(self.operations_called[1][0].name, 'UploadPart')
        self.assertEqual(self.operations_called[2][0].name, 'UploadPart')
        self.assertEqual(
            self.operations_called[3][0].name, 'CompleteMultipartUpload'
        )
        # Verify the IfNoneMatch condition was set in the CompleteMultipartUpload request
        self.assertEqual(self.operations_called[3][1]['IfNoneMatch'], '*')
        # Verify source file was deleted (successful move operation)
        self.assertFalse(os.path.exists(full_path))

    def test_mv_no_overwrite_flag_multipart_upload_when_object_exists_on_target(
        self,
    ):
        # Create a large file that will trigger multipart upload
        full_path = self.files.create_file('foo.txt', 'a' * 10 * (1024**2))
        cmdline = f'{self.prefix} {full_path} s3://bucket --no-overwrite'
        # Set up responses for multipart upload
        self.parsed_responses = [
            {'UploadId': 'foo'},  # CreateMultipartUpload response
            {'ETag': '"foo-1"'},  # UploadPart response
            {'ETag': '"foo-2"'},  # UploadPart response
            self.precondition_failed_error_response(),  # CompleteMultipartUpload response
            {},  # Abort Multipart
        ]
        self.run_cmd(cmdline, expected_rc=0)
        # Set up the response to simulate a PreconditionFailed error
        self.http_response.status_code = 412
        # Verify all multipart operations were called
        self.assertEqual(len(self.operations_called), 5)
        self.assertEqual(
            self.operations_called[0][0].name, 'CreateMultipartUpload'
        )
        self.assertEqual(self.operations_called[1][0].name, 'UploadPart')
        self.assertEqual(self.operations_called[2][0].name, 'UploadPart')
        self.assertEqual(
            self.operations_called[3][0].name, 'CompleteMultipartUpload'
        )
        self.assertEqual(
            self.operations_called[4][0].name, 'AbortMultipartUpload'
        )
        # Verify the IfNoneMatch condition was set in the CompleteMultipartUpload request
        self.assertEqual(self.operations_called[3][1]['IfNoneMatch'], '*')
        # Verify source file was not deleted (failed move operation due to PreconditionFailed)
        self.assertTrue(os.path.exists(full_path))

    def test_mv_no_overwrite_flag_on_copy_when_small_object_does_not_exist_on_target(
        self,
    ):
        cmdline = f'{self.prefix} s3://bucket1/key.txt s3://bucket2/key1.txt --no-overwrite'
        # Set up responses for multipart copy (since no-overwrite always uses multipart)
        self.parsed_responses = [
            self.head_object_response(),  # HeadObject to get source metadata
            self.copy_object_response(),
            self.delete_object_response(),
        ]
        self.run_cmd(cmdline, expected_rc=0)
        # Verify all multipart copy operations were called
        self.assertEqual(len(self.operations_called), 3)
        self.assertEqual(self.operations_called[0][0].name, 'HeadObject')
        self.assertEqual(self.operations_called[1][0].name, 'CopyObject')
        self.assertEqual(self.operations_called[1][1]['IfNoneMatch'], '*')

        self.assertEqual(self.operations_called[2][0].name, 'DeleteObject')

    def test_mv_no_overwrite_flag_on_copy_when_small_object_exists_on_target(
        self,
    ):
        cmdline = f'{self.prefix} s3://bucket1/key.txt s3://bucket2/key.txt --no-overwrite'
        # Set up responses for multipart copy (since no-overwrite always uses multipart)
        self.parsed_responses = [
            self.head_object_response(),  # HeadObject to get source metadata
            self.precondition_failed_error_response(),  # CopyObject response
        ]
        self.run_cmd(cmdline, expected_rc=0)
        # Set up the response to simulate a PreconditionFailed error
        self.http_response.status_code = 412
        # Verify all copy operations were called
        self.assertEqual(len(self.operations_called), 2)
        self.assertEqual(self.operations_called[0][0].name, 'HeadObject')
        self.assertEqual(self.operations_called[1][0].name, 'CopyObject')
        # Verify the IfNoneMatch condition was set in the CopyObject request
        self.assertEqual(self.operations_called[1][1]['IfNoneMatch'], '*')

    def test_mv_no_overwrite_flag_when_large_object_exists_on_target(self):
        cmdline = f'{self.prefix} s3://bucket1/key1.txt s3://bucket/key1.txt --no-overwrite'
        self.parsed_responses = [
            self.head_object_response(ContentLength=10 * (1024**2)),
            self.get_object_tagging_response({}),  # GetObjectTagging response
            self.create_mpu_response('foo'),  # CreateMultipartUpload response
            self.upload_part_copy_response(),  # UploadPartCopy response part 1
            self.upload_part_copy_response(),  # UploadPartCopy response part 2
            self.precondition_failed_error_response(),  # CompleteMultipartUpload fails with PreconditionFailed
            {},  # AbortMultipartUpload response
        ]
        self.run_cmd(cmdline, expected_rc=0)
        # Verify all multipart operations were called
        self.assertEqual(len(self.operations_called), 7)
        self.assertEqual(self.operations_called[0][0].name, 'HeadObject')
        self.assertEqual(self.operations_called[1][0].name, 'GetObjectTagging')
        self.assertEqual(
            self.operations_called[2][0].name, 'CreateMultipartUpload'
        )
        self.assertEqual(self.operations_called[3][0].name, 'UploadPartCopy')
        self.assertEqual(self.operations_called[4][0].name, 'UploadPartCopy')
        self.assertEqual(
            self.operations_called[5][0].name, 'CompleteMultipartUpload'
        )
        self.assertEqual(
            self.operations_called[6][0].name, 'AbortMultipartUpload'
        )
        # Verify the IfNoneMatch condition was set in the CompleteMultipartUpload request
        self.assertEqual(self.operations_called[5][1]['IfNoneMatch'], '*')

    def test_mv_no_overwrite_flag_when_large_object_does_not_exist_on_target(
        self,
    ):
        cmdline = f'{self.prefix} s3://bucket1/key1.txt s3://bucket/key.txt --no-overwrite'
        self.parsed_responses = [
            self.head_object_response(ContentLength=10 * (1024**2)),
            self.get_object_tagging_response({}),  # GetObjectTagging response
            self.create_mpu_response('foo'),  # CreateMultipartUpload response
            self.upload_part_copy_response(),  # UploadPartCopy response part 1
            self.upload_part_copy_response(),  # UploadPartCopy response part 2
            {},  # CompleteMultipartUpload response
            self.delete_object_response(),  # DeleteObject (for move operation)
        ]
        self.run_cmd(cmdline, expected_rc=0)
        # Verify all multipart operations were called
        self.assertEqual(len(self.operations_called), 7)
        self.assertEqual(self.operations_called[0][0].name, 'HeadObject')
        self.assertEqual(self.operations_called[1][0].name, 'GetObjectTagging')
        self.assertEqual(
            self.operations_called[2][0].name, 'CreateMultipartUpload'
        )
        self.assertEqual(self.operations_called[3][0].name, 'UploadPartCopy')
        self.assertEqual(self.operations_called[4][0].name, 'UploadPartCopy')
        self.assertEqual(
            self.operations_called[5][0].name, 'CompleteMultipartUpload'
        )
        self.assertEqual(self.operations_called[6][0].name, 'DeleteObject')

    def test_no_overwrite_flag_on_mv_download_when_single_object_exists_at_target(
        self,
    ):
        full_path = self.files.create_file('foo.txt', 'existing content')
        cmdline = (
            f'{self.prefix} s3://bucket/foo.txt {full_path} --no-overwrite'
        )
        self.parsed_responses = [
            self.head_object_response(),
        ]
        self.run_cmd(cmdline, expected_rc=0)
        self.assertEqual(len(self.operations_called), 1)
        self.assertEqual(self.operations_called[0][0].name, 'HeadObject')
        with open(full_path) as f:
            self.assertEqual(f.read(), 'existing content')

    def test_no_overwrite_flag_on_mv_download_when_single_object_does_not_exist_at_target(
        self,
    ):
        full_path = self.files.full_path('foo.txt')
        cmdline = (
            f'{self.prefix} s3://bucket/foo.txt {full_path} --no-overwrite'
        )
        self.parsed_responses = [
            self.head_object_response(),
            self.get_object_response(),
            self.delete_object_response(),
        ]
        self.run_cmd(cmdline, expected_rc=0)
        self.assertEqual(len(self.operations_called), 3)
        self.assertEqual(self.operations_called[0][0].name, 'HeadObject')
        self.assertEqual(self.operations_called[1][0].name, 'GetObject')
        self.assertEqual(self.operations_called[2][0].name, 'DeleteObject')
        with open(full_path) as f:
            self.assertEqual(f.read(), 'foo')


class TestMvWithCRTClient(BaseCRTTransferClientTest):
    def test_upload_move_using_crt_client(self):
        filename = self.files.create_file('myfile', 'mycontent')
        cmdline = [
            's3',
            'mv',
            filename,
            's3://bucket/key',
        ]
        self.run_command(cmdline)
        crt_requests = self.get_crt_make_request_calls()
        self.assertEqual(len(crt_requests), 1)
        self.assert_crt_make_request_call(
            crt_requests[0],
            expected_type=S3RequestType.PUT_OBJECT,
            expected_host=self.get_virtual_s3_host('bucket'),
            expected_path='/key',
            expected_send_filepath=filename,
        )
        self.assertFalse(os.path.exists(filename))

    def test_download_move_using_crt_client(self):
        filename = os.path.join(self.files.rootdir, 'myfile')
        cmdline = ['s3', 'mv', 's3://bucket/key', filename]
        self.add_botocore_head_object_response()
        self.add_botocore_delete_object_response()
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        crt_requests = self.get_crt_make_request_calls()
        self.assertEqual(len(crt_requests), 1)
        self.assert_crt_make_request_call(
            crt_requests[0],
            expected_type=S3RequestType.GET_OBJECT,
            expected_host=self.get_virtual_s3_host('bucket'),
            expected_path='/key',
            expected_recv_startswith=filename,
        )
        self.assert_no_remaining_botocore_responses()

    def test_does_not_use_crt_client_for_copy_moves(self):
        cmdline = ['s3', 'mv', 's3://bucket/key', 's3://otherbucket/']
        self.add_botocore_head_object_response()
        self.add_botocore_copy_object_response()
        self.add_botocore_delete_object_response()
        self.run_command(cmdline)
        self.assertEqual(self.get_crt_make_request_calls(), [])
        self.assert_no_remaining_botocore_responses()


class TestMvCommandWithValidateSameS3Paths(BaseS3TransferCommandTest):
    prefix = 's3 mv '

    def assert_validates_cannot_mv_onto_itself(self, cmd):
        stderr = self.run_cmd(cmd, expected_rc=252)[1]
        self.assertIn('Cannot mv a file onto itself', stderr)

    def assert_runs_mv_without_validation(self, cmd):
        self.parsed_responses = [
            self.head_object_response(),
            self.copy_object_response(),
            self.delete_object_response(),
        ]
        self.run_cmd(cmd, expected_rc=0)
        self.assertEqual(
            len(self.operations_called), 3, self.operations_called
        )
        self.assertEqual(self.operations_called[0][0].name, 'HeadObject')
        self.assertEqual(self.operations_called[1][0].name, 'CopyObject')
        self.assertEqual(self.operations_called[2][0].name, 'DeleteObject')

    def assert_raises_warning(self, cmd):
        self.parsed_responses = [
            self.head_object_response(),
            self.copy_object_response(),
            self.delete_object_response(),
        ]
        stderr = self.run_cmd(cmd, expected_rc=0)[1]
        self.assertIn('warning: Provided s3 paths may resolve', stderr)

    def test_cant_mv_object_onto_itself_access_point_arn(self):
        cmdline = (
            f"{self.prefix}s3://bucket/key "
            "s3://arn:aws:s3:us-west-2:123456789012:accesspoint/"
            "myaccesspoint/key "
            "--validate-same-s3-paths"
        )
        self.parsed_responses = [{"Bucket": "bucket"}]
        self.assert_validates_cannot_mv_onto_itself(cmdline)

    def test_cant_mv_object_onto_itself_access_point_arn_as_source(self):
        cmdline = (
            f"{self.prefix}s3://arn:aws:s3:us-west-2:123456789012:"
            "accesspoint/myaccesspoint/key "
            "s3://bucket/key "
            "--validate-same-s3-paths"
        )
        self.parsed_responses = [{"Bucket": "bucket"}]
        self.assert_validates_cannot_mv_onto_itself(cmdline)

    def test_cant_mv_object_onto_itself_access_point_arn_with_env_var(self):
        self.environ['AWS_CLI_S3_MV_VALIDATE_SAME_S3_PATHS'] = 'true'
        cmdline = (
            f"{self.prefix}s3://bucket/key "
            "s3://arn:aws:s3:us-west-2:123456789012:accesspoint/"
            "myaccesspoint/key"
        )
        self.parsed_responses = [{"Bucket": "bucket"}]
        self.assert_validates_cannot_mv_onto_itself(cmdline)

    def test_cant_mv_object_onto_itself_access_point_arn_base_key(self):
        cmdline = (
            f"{self.prefix}s3://bucket/key "
            "s3://arn:aws:s3:us-west-2:123456789012:accesspoint/"
            "myaccesspoint/ "
            "--validate-same-s3-paths"
        )
        self.parsed_responses = [{"Bucket": "bucket"}]
        self.assert_validates_cannot_mv_onto_itself(cmdline)

    def test_cant_mv_object_onto_itself_access_point_arn_base_prefix(self):
        cmdline = (
            f"{self.prefix}s3://bucket/prefix/key "
            "s3://arn:aws:s3:us-west-2:123456789012:accesspoint/"
            "myaccesspoint/prefix/ "
            "--validate-same-s3-paths"
        )
        self.parsed_responses = [{"Bucket": "bucket"}]
        self.assert_validates_cannot_mv_onto_itself(cmdline)

    def test_cant_mv_object_onto_itself_access_point_alias(self):
        cmdline = (
            f"{self.prefix} s3://bucket/key "
            "s3://myaccesspoint-foobar-s3alias/key "
            "--validate-same-s3-paths"
        )
        self.parsed_responses = [
            {"Account": "123456789012"},
            {"Bucket": "bucket"},
        ]
        self.assert_validates_cannot_mv_onto_itself(cmdline)

    def test_cant_mv_object_onto_itself_outpost_access_point_arn(self):
        cmdline = (
            f"{self.prefix}s3://bucket/key "
            "s3://arn:aws:s3-outposts:us-east-1:123456789012:outpost/"
            "op-foobar/accesspoint/myaccesspoint/key "
            "--validate-same-s3-paths"
        )
        self.parsed_responses = [{"Bucket": "bucket"}]
        self.assert_validates_cannot_mv_onto_itself(cmdline)

    def test_outpost_access_point_alias_raises_error(self):
        cmdline = (
            f"{self.prefix} s3://bucket/key "
            "s3://myaccesspoint-foobar--op-s3/key "
            "--validate-same-s3-paths"
        )
        stderr = self.run_cmd(cmdline, expected_rc=252)[1]
        self.assertIn("Can't resolve underlying bucket name", stderr)

    def test_cant_mv_object_onto_itself_mrap_arn(self):
        cmdline = (
            f"{self.prefix} s3://bucket/key "
            "s3://arn:aws:s3::123456789012:accesspoint/foobar.mrap/key "
            "--validate-same-s3-paths"
        )
        self.parsed_responses = [
            {
                "AccessPoints": [
                    {
                        "Alias": "foobar.mrap",
                        "Regions": [
                            {"Bucket": "differentbucket"},
                            {"Bucket": "bucket"},
                        ],
                    }
                ]
            }
        ]
        self.assert_validates_cannot_mv_onto_itself(cmdline)

    def test_get_mrap_buckets_raises_if_alias_not_found(self):
        cmdline = (
            f"{self.prefix} s3://bucket/key "
            "s3://arn:aws:s3::123456789012:accesspoint/foobar.mrap/key "
            "--validate-same-s3-paths"
        )
        self.parsed_responses = [
            {
                "AccessPoints": [
                    {
                        "Alias": "baz.mrap",
                        "Regions": [
                            {"Bucket": "differentbucket"},
                            {"Bucket": "bucket"},
                        ],
                    }
                ]
            }
        ]
        stderr = self.run_cmd(cmdline, expected_rc=252)[1]
        self.assertEqual(
            "\naws: [ERROR]: An error occurred (ParamValidation): "
            "Couldn't find multi-region access point with alias foobar.mrap "
            "in account 123456789012\n",
            stderr,
        )

    def test_mv_works_if_access_point_arn_resolves_to_different_bucket(self):
        cmdline = (
            f"{self.prefix}s3://bucket/key "
            "s3://arn:aws:s3:us-west-2:123456789012:accesspoint/"
            "myaccesspoint/key "
            "--validate-same-s3-paths"
        )
        self.parsed_responses = [
            {"Bucket": "differentbucket"},
            self.head_object_response(),
            self.copy_object_response(),
            self.delete_object_response(),
        ]
        self.run_cmd(cmdline, expected_rc=0)
        self.assertEqual(
            len(self.operations_called), 4, self.operations_called
        )
        self.assertEqual(self.operations_called[0][0].name, 'GetAccessPoint')
        self.assertEqual(self.operations_called[1][0].name, 'HeadObject')
        self.assertEqual(self.operations_called[2][0].name, 'CopyObject')
        self.assertEqual(self.operations_called[3][0].name, 'DeleteObject')

    def test_mv_works_if_access_point_alias_resolves_to_different_bucket(self):
        cmdline = (
            f"{self.prefix} s3://bucket/key "
            "s3://myaccesspoint-foobar-s3alias/key "
            "--validate-same-s3-paths"
        )
        self.parsed_responses = [
            {"Account": "123456789012"},
            {"Bucket": "differentbucket"},
            self.head_object_response(),
            self.copy_object_response(),
            self.delete_object_response(),
        ]
        self.run_cmd(cmdline, expected_rc=0)
        self.assertEqual(
            len(self.operations_called), 5, self.operations_called
        )
        self.assertEqual(
            self.operations_called[0][0].name, 'GetCallerIdentity'
        )
        self.assertEqual(self.operations_called[1][0].name, 'GetAccessPoint')
        self.assertEqual(self.operations_called[2][0].name, 'HeadObject')
        self.assertEqual(self.operations_called[3][0].name, 'CopyObject')
        self.assertEqual(self.operations_called[4][0].name, 'DeleteObject')

    def test_mv_works_if_outpost_access_point_arn_resolves_to_different_bucket(
        self,
    ):
        cmdline = (
            f"{self.prefix}s3://bucket/key "
            "s3://arn:aws:s3-outposts:us-east-1:123456789012:outpost/"
            "op-foobar/accesspoint/myaccesspoint/key "
            "--validate-same-s3-paths"
        )
        self.parsed_responses = [
            {"Bucket": "differentbucket"},
            self.head_object_response(),
            self.copy_object_response(),
            self.delete_object_response(),
        ]
        self.run_cmd(cmdline, expected_rc=0)
        self.assertEqual(
            len(self.operations_called), 4, self.operations_called
        )
        self.assertEqual(self.operations_called[0][0].name, 'GetAccessPoint')
        self.assertEqual(self.operations_called[1][0].name, 'HeadObject')
        self.assertEqual(self.operations_called[2][0].name, 'CopyObject')
        self.assertEqual(self.operations_called[3][0].name, 'DeleteObject')

    @requires_crt()
    def test_mv_works_if_mrap_arn_resolves_to_different_bucket(self):
        cmdline = (
            f"{self.prefix} s3://bucket/key "
            "s3://arn:aws:s3::123456789012:accesspoint/foobar.mrap/key "
            "--validate-same-s3-paths"
        )
        self.parsed_responses = [
            {
                "AccessPoints": [
                    {
                        "Alias": "foobar.mrap",
                        "Regions": [
                            {"Bucket": "differentbucket"},
                        ],
                    }
                ]
            },
            self.head_object_response(),
            self.copy_object_response(),
            self.delete_object_response(),
        ]
        self.run_cmd(cmdline, expected_rc=0)
        self.assertEqual(
            len(self.operations_called), 4, self.operations_called
        )
        self.assertEqual(
            self.operations_called[0][0].name, 'ListMultiRegionAccessPoints'
        )
        self.assertEqual(self.operations_called[1][0].name, 'HeadObject')
        self.assertEqual(self.operations_called[2][0].name, 'CopyObject')
        self.assertEqual(self.operations_called[3][0].name, 'DeleteObject')

    def test_skips_validation_if_keys_are_different_accesspoint_arn(self):
        cmdline = (
            f"{self.prefix}s3://bucket/key "
            "s3://arn:aws:s3:us-west-2:123456789012:accesspoint/"
            "myaccesspoint/key2 "
            "--validate-same-s3-paths"
        )
        self.assert_runs_mv_without_validation(cmdline)

    def test_skips_validation_if_prefixes_are_different_accesspoint_arn(self):
        cmdline = (
            f"{self.prefix}s3://bucket/key "
            "s3://arn:aws:s3:us-west-2:123456789012:accesspoint/"
            "myaccesspoint/prefix/ "
            "--validate-same-s3-paths"
        )
        self.assert_runs_mv_without_validation(cmdline)

    def test_skips_validation_if_keys_are_different_accesspoint_alias(self):
        cmdline = (
            f"{self.prefix} s3://bucket/key "
            "s3://myaccesspoint-foobar-s3alias/key2 "
            "--validate-same-s3-paths"
        )
        self.assert_runs_mv_without_validation(cmdline)

    def test_skips_validation_if_keys_are_different_outpost_arn(self):
        cmdline = (
            f"{self.prefix}s3://bucket/key "
            "s3://arn:aws:s3-outposts:us-east-1:123456789012:outpost/"
            "op-foobar/accesspoint/myaccesspoint/key2 "
            "--validate-same-s3-paths"
        )
        self.assert_runs_mv_without_validation(cmdline)

    def test_skips_validation_if_keys_are_different_outpost_alias(self):
        cmdline = (
            f"{self.prefix} s3://bucket/key "
            "s3://myaccesspoint-foobar--op-s3/key2 "
            "--validate-same-s3-paths"
        )
        self.assert_runs_mv_without_validation(cmdline)

    @requires_crt()
    def test_skips_validation_if_keys_are_different_mrap_arn(self):
        cmdline = (
            f"{self.prefix} s3://bucket/key "
            "s3://arn:aws:s3::123456789012:accesspoint/foobar.mrap/key2 "
            "--validate-same-s3-paths"
        )
        self.assert_runs_mv_without_validation(cmdline)

    def test_raises_warning_if_validation_not_set(self):
        cmdline = (
            f"{self.prefix}s3://bucket/key "
            "s3://arn:aws:s3:us-west-2:123456789012:accesspoint/"
            "myaccesspoint/key"
        )
        self.assert_raises_warning(cmdline)

    def test_raises_warning_if_validation_not_set_source(self):
        cmdline = (
            f"{self.prefix}"
            "s3://arn:aws:s3:us-west-2:123456789012:accesspoint/"
            "myaccesspoint/key "
            "s3://bucket/key"
        )
        self.assert_raises_warning(cmdline)


class TestMvRecursiveCaseConflict(TestSyncCaseConflict):
    prefix = 's3 mv --recursive '

    @skip_if_case_sensitive()
    def test_warn_with_existing_file(self):
        self.files.create_file(self.lower_key, 'mycontent')
        cmd = (
            f"{self.prefix} s3://bucket {self.files.rootdir} "
            "--case-conflict warn"
        )
        self.parsed_responses = [
            self.list_objects_response([self.upper_key]),
            self.get_object_response(),
            self.delete_object_response(),
        ]
        _, stderr, _ = self.run_cmd(cmd, expected_rc=0)
        assert f"warning: Downloading bucket/{self.upper_key}" in stderr

    def test_warn_with_case_conflicts_in_s3(self):
        # This test case becomes very flaky because mv
        # performs a get and delete operation twice.
        # Delete is called after the get finishes, but
        # the order of responses become non-deterministic
        # when downloading multiple objects. The order
        # could be [get, get, delete, delete] or
        # [get, delete, get, delete]. Rather than making
        # complex changes to patch this behavior, we're
        # delegating the assertions to the sync and cp
        # test suites.
        pass

    def test_skip_with_case_conflicts_in_s3(self):
        cmd = (
            f"{self.prefix} s3://bucket {self.files.rootdir} "
            "--case-conflict skip"
        )
        self.parsed_responses = [
            self.list_objects_response([self.upper_key, self.lower_key]),
            self.get_object_response(),
            self.delete_object_response(),
        ]
        _, stderr, _ = self.run_cmd(cmd, expected_rc=0)
        assert f"warning: Skipping bucket/{self.lower_key}" in stderr

    def test_ignore_with_existing_file(self):
        self.files.create_file(self.lower_key, 'mycontent')
        cmd = (
            f"{self.prefix} s3://bucket {self.files.rootdir} "
            "--case-conflict ignore"
        )
        self.parsed_responses = [
            self.list_objects_response([self.upper_key]),
            self.get_object_response(),
            self.delete_object_response(),
        ]
        self.run_cmd(cmd, expected_rc=0)

    def test_ignore_with_case_conflicts_in_s3(self):
        pass


class TestS3ExpressMvRecursive(BaseS3TransferCommandTest):
    prefix = 's3 mv --recursive '

    def test_s3_express_error_raises_exception(self):
        cmd = (
            f"{self.prefix} s3://bucket--usw2-az1--x-s3 {self.files.rootdir} "
            "--case-conflict error"
        )
        _, stderr, _ = self.run_cmd(cmd, expected_rc=252)
        assert "`error` is not a valid value" in stderr

    def test_s3_express_skip_raises_exception(self):
        cmd = (
            f"{self.prefix} s3://bucket--usw2-az1--x-s3 {self.files.rootdir} "
            "--case-conflict skip"
        )
        _, stderr, _ = self.run_cmd(cmd, expected_rc=252)
        assert "`skip` is not a valid value" in stderr



# Copyright 2021 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import base64
import hashlib
import os
import re
import shutil
from pathlib import Path

import py
import pytest

import awscli
import backends.pep517


@pytest.fixture
def config_settings():
    return {}


@pytest.fixture
def repo_root():
    return str(Path(__file__).parents[2])


@pytest.fixture
def preserve_ac_index(repo_root, tmpdir):
    # Some functional tests will build the ac.index in-place.
    # In case there was an existing one we will need to move it out of the
    # way and then back once the test is complete.
    # If thre is no ac.index file, we need to clean up the one generated
    # by the tests.
    ac_index_path = os.path.join(repo_root, "awscli", "data", "ac.index")
    save_location = tmpdir.join("ac.index")
    if os.path.isfile(ac_index_path):
        os.rename(ac_index_path, save_location)
    yield
    if os.path.isfile(save_location):
        os.rename(save_location, ac_index_path)
    else:
        os.remove(ac_index_path)


@pytest.fixture(autouse=True)
def cd_to_repo_root(repo_root):
    # In general, we want to be at the root of the repository when running
    # PEP 517 hooks.
    original = os.getcwd()
    os.chdir(repo_root)
    yield
    os.chdir(original)


def assert_dist_info_correctness(dist_info_dir):
    # A helper utility to do sanity checks on the generated wheel .dist-info
    # directory. Right now we only look at the generated METADATA file,
    # but this could be expanded to other files that are expected to be in
    # the .dist-info directory
    metadata_file = dist_info_dir.join("METADATA")
    assert metadata_file.check()
    metadata_content = metadata_file.read()
    assert "Metadata-Version: 2.4" in metadata_content
    assert "Name: awscli" in metadata_content
    assert f"Version: {awscli.__version__}" in metadata_content


def assert_dependency_in_requirements(package_name, requirements):
    pattern = re.compile(rf"^{package_name}.*$")
    for req in requirements:
        if pattern.match(req):
            return
    raise AssertionError(
        f'Could not find package: "{package_name}" in '
        f"requirements: {requirements}"
    )


def test_build_sdist(tmpdir, config_settings):
    sdist_name = backends.pep517.build_sdist(str(tmpdir), config_settings)
    expected_sdist_name = f"awscli-{awscli.__version__}.tar.gz"
    assert sdist_name == expected_sdist_name

    path_to_sdist = tmpdir.join(sdist_name)
    assert path_to_sdist.check()

    # Unpack the tarball and do some sanity checks to make sure the
    # sdist looks correct.
    shutil.unpack_archive(path_to_sdist, tmpdir)
    unpacked_sdist = tmpdir.join(f"awscli-{awscli.__version__}")

    assert unpacked_sdist.join("pyproject.toml").check()
    assert unpacked_sdist.join("awscli", "__init__.py").check()
    assert unpacked_sdist.join("CHANGELOG.rst").check()

    # Make sure the backends package is in the sdist so users can build
    # a wheel from an sdist using PEP 517.
    assert unpacked_sdist.join("backends", "pep517.py").check()

    # Make sure the bin directory is included.
    assert unpacked_sdist.join("bin", "aws").check()

    # Make sure the tests directory is included.
    assert unpacked_sdist.join("tests", "__init__.py").check()
    assert unpacked_sdist.join("tests", "unit", "__init__.py").check()
    assert unpacked_sdist.join(
        "tests", "backends", "build_system", "unit", "__init__.py"
    ).check()
    assert unpacked_sdist.join("tests", "__init__.py").check()

    # Make sure sdist will be buildable
    assert unpacked_sdist.join("configure").check()
    assert unpacked_sdist.join("Makefile.in").check()
    assert unpacked_sdist.join("requirements", "bootstrap.txt").check()
    assert unpacked_sdist.join(
        "requirements", "download-deps", "bootstrap.txt"
    ).check()

    # Make sure exe build files are added to the sdist
    assert unpacked_sdist.join("exe", "pyinstaller", "aws.spec")
    assert unpacked_sdist.join("exe", "asssets", "install")
    assert unpacked_sdist.join("exe", "tests", "README.md")

    # We do not build the ac.index in building the sdist. So we want to make
    # sure it is not being included.
    assert not unpacked_sdist.join("awscli", "data", "ac.index").check()


def test_build_wheel(tmpdir, config_settings, repo_root):
    wheel_name = backends.pep517.build_wheel(str(tmpdir), config_settings)
    expected_wheel_name = f"awscli-{awscli.__version__}-py3-none-any.whl"
    assert wheel_name == expected_wheel_name

    path_to_wheel = tmpdir.join(wheel_name)
    assert path_to_wheel.check()

    # Unpack the wheel and do some sanity checks to make sure the
    # sdist looks correct.
    unpacked_wheel_dir = tmpdir.join("unpacked_wheel")
    unpacked_wheel_dist_info = unpacked_wheel_dir.join(
        f"awscli-{awscli.__version__}.dist-info"
    )
    shutil.unpack_archive(path_to_wheel, unpacked_wheel_dir, "zip")
    assert unpacked_wheel_dir.join("awscli", "__init__.py").check()
    assert_dist_info_correctness(unpacked_wheel_dist_info)

    # Make sure scripts were included and the shebang got re-written
    # correctly.
    unpacked_wheel_data = unpacked_wheel_dir.join(
        f"awscli-{awscli.__version__}.data"
    )
    assert unpacked_wheel_data.join("scripts", "aws").check()
    assert "#!python" in unpacked_wheel_data.join("scripts", "aws").read()

    # We do not want the backends package included as part of the
    # environment's site-packages so it should not exist in the wheel
    assert not unpacked_wheel_dir.join("backends", "pep517.py").check()

    # The wheel should have built the auto-complete index and repacked it
    # into the wheel.
    assert unpacked_wheel_dir.join("awscli", "data", "ac.index").check()
    records_content = unpacked_wheel_dist_info.join("RECORD").read()
    assert "/".join(["awscli", "data", "ac.index"]) in records_content

    _create_record_file(
        unpacked_wheel_dir, os.path.split(unpacked_wheel_dist_info)[-1]
    )
    expected_records_content = unpacked_wheel_dist_info.join("RECORD").read()
    assert records_content == expected_records_content


def test_build_editable(tmpdir, config_settings, repo_root, preserve_ac_index):
    wheel_name = backends.pep517.build_editable(str(tmpdir), config_settings)
    path_to_wheel = tmpdir.join(wheel_name)

    unpacked_wheel_dir = tmpdir.join("unpacked_wheel")
    unpacked_wheel_dist_info = unpacked_wheel_dir.join(
        f"awscli-{awscli.__version__}.dist-info"
    )
    unpacked_wheel_data = unpacked_wheel_dir.join(
        f"awscli-{awscli.__version__}.data"
    )
    shutil.unpack_archive(path_to_wheel, unpacked_wheel_dir, "zip")

    assert unpacked_wheel_dist_info.check()
    assert unpacked_wheel_data.check()
    assert unpacked_wheel_data.join("scripts").join("aws").check()
    assert unpacked_wheel_data.join("scripts").join("aws_completer").check()

    # Check that the ac.index got created in the base repo. The
    # relocate_ac_index fixture moves the existing one out of the repo before
    # this tests runs so if the file exists it is because it was generated by
    # backends.pep517.build_editable.
    assert (
        py.path.local(repo_root)
        .join("awscli")
        .join("data")
        .join("ac.index")
        .check()
    )

    pth_file = unpacked_wheel_dir.join("awscli.pth")
    assert pth_file.check()
    assert pth_file.read() == repo_root


def test_prepare_metadata_for_build_wheel(tmpdir, config_settings):
    dist_info_dirname = backends.pep517.prepare_metadata_for_build_wheel(
        str(tmpdir), config_settings
    )
    assert dist_info_dirname.startswith("awscli")
    assert dist_info_dirname.endswith(".dist-info")
    assert_dist_info_correctness(tmpdir.join(dist_info_dirname))


def test_prepare_metadata_for_build_editable(tmpdir, config_settings):
    dist_info_dirname = backends.pep517.prepare_metadata_for_build_editable(
        str(tmpdir), config_settings
    )
    assert dist_info_dirname.startswith("awscli")
    assert dist_info_dirname.endswith(".dist-info")
    assert_dist_info_correctness(tmpdir.join(dist_info_dirname))


def test_get_requires_for_build_sdist(config_settings):
    requirements = backends.pep517.get_requires_for_build_sdist(
        config_settings
    )
    assert requirements == []


def test_get_requires_for_build_wheel(config_settings, repo_root):
    requirements = backends.pep517.get_requires_for_build_wheel(
        config_settings
    )
    expected_requirements = [
        "colorama",
        "docutils",
        "ruamel_yaml",
        "ruamel_yaml_clib",
        "prompt-toolkit",
        "distro",
        "awscrt",
        "python-dateutil",
        "jmespath",
        "urllib3",
        "wcwidth",
    ]
    assert len(expected_requirements) == len(requirements)
    for expected_requirement in expected_requirements:
        assert_dependency_in_requirements(expected_requirement, requirements)


def test_read_sdist_extras():
    expected_extras = {
        "backends/**/*.py",
        "bin/*",
        "CHANGELOG.rst",
        "tests/**/*",
        "requirements/**/*.txt",
        "configure",
        "Makefile.in",
        "exe/**/*",
    }
    extras = set(backends.pep517.read_sdist_extras())

    assert extras == expected_extras


# Since we regenerate the RECORD file in the wheel directory this is a
# duplicate of the logic from that file. At the time of writing the
# RECORD file in the wheel is the same as the originally generated one
# using setuptools sans the metadata file.
# Comparing the generated wheel RECORD file to one generated by this
# snippet should alert us if we break anything in the pep517 module.
def _create_record_file(base_dir, dist_info_dir):
    entries = []
    for base, dirs, files in os.walk(base_dir):
        dirs.sort()
        for filename in sorted(files):
            # Record file has its own special entry at the end
            if filename == "RECORD":
                continue
            target = os.path.join(base, filename)
            # Path entries in record file are unix style
            path = os.path.relpath(target, base_dir).replace(os.sep, "/")
            digest = _get_b64_sha256(target)
            size = os.path.getsize(target)
            record_entry = f"{path},sha256={digest},{size}"
            entries.append(record_entry)
    entries.append(f"{dist_info_dir}/RECORD,,")
    record_path = os.path.join(base_dir, dist_info_dir, "RECORD")
    with open(record_path, "w") as f:
        f.write("\n".join(entries) + "\n")


def _get_b64_sha256(path):
    with open(path, "rb") as f:
        file_bytes = f.read()
        digest_bytes = hashlib.sha256(file_bytes).digest()
    encoded = base64.urlsafe_b64encode(digest_bytes).rstrip(b"=")
    return encoded.decode("utf-8")



#!/usr/bin/env python
# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os
import re

import awscli.clidriver
from awscli.compat import StringIO
from awscli.testutils import BaseAWSCommandParamsTest


class TestGetObject(BaseAWSCommandParamsTest):
    prefix = 's3api get-object'

    def setUp(self):
        super(TestGetObject, self).setUp()
        self.parsed_response = {'Body': StringIO()}

    def remove_file_if_exists(self, filename):
        if os.path.isfile(filename):
            os.remove(filename)

    def test_simple(self):
        cmdline = self.prefix
        cmdline += ' --bucket mybucket'
        cmdline += ' --key mykey'
        cmdline += ' outfile'
        self.addCleanup(self.remove_file_if_exists, 'outfile')
        self.assert_params_for_cmd(
            cmdline,
            {'Bucket': 'mybucket', 'ChecksumMode': 'ENABLED', 'Key': 'mykey'},
        )

    def test_range(self):
        cmdline = self.prefix
        cmdline += ' --bucket mybucket'
        cmdline += ' --key mykey'
        cmdline += ' --range bytes=0-499'
        cmdline += ' outfile'
        self.addCleanup(self.remove_file_if_exists, 'outfile')
        self.assert_params_for_cmd(
            cmdline,
            {
                'Bucket': 'mybucket',
                'ChecksumMode': 'ENABLED',
                'Key': 'mykey',
                'Range': 'bytes=0-499',
            },
        )

    def test_response_headers(self):
        cmdline = self.prefix
        cmdline += ' --bucket mybucket'
        cmdline += ' --key mykey'
        cmdline += ' --response-cache-control No-cache'
        cmdline += ' --response-content-encoding x-gzip'
        cmdline += ' outfile'
        self.addCleanup(self.remove_file_if_exists, 'outfile')
        self.assert_params_for_cmd(
            cmdline,
            {
                'Bucket': 'mybucket',
                'ChecksumMode': 'ENABLED',
                'Key': 'mykey',
                'ResponseCacheControl': 'No-cache',
                'ResponseContentEncoding': 'x-gzip',
            },
        )

    def test_streaming_output_arg_with_error_response(self):
        # Checking that the StreamingOutputArg handles the
        # case where it's passed an error body.  Previously
        # it would propogate a KeyError so we want to ensure
        # this case is handled.
        self.parsed_response = {
            'Error': {'Code': 'AuthError', 'Message': 'SomeError'}
        }
        cmdline = self.prefix
        cmdline += ' --bucket mybucket'
        cmdline += ' --key mykey'
        cmdline += ' outfile'
        self.addCleanup(self.remove_file_if_exists, 'outfile')
        self.assert_params_for_cmd(
            cmdline,
            {'Bucket': 'mybucket', 'ChecksumMode': 'ENABLED', 'Key': 'mykey'},
        )


if __name__ == "__main__":
    unittest.main()



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import base64
import json
from datetime import datetime

import botocore

from awscli.customizations.eks.get_token import GetTokenCommand, TokenGenerator
from awscli.testutils import capture_output, mock, unittest


class BaseTokenTest(unittest.TestCase):
    def setUp(self):
        self._session = botocore.session.get_session()
        self._access_key = 'ABCDEFGHIJKLMNOPQRST'
        self._secret_key = 'TSRQPONMLKJUHGFEDCBA'
        self._region = 'us-west-2'
        self._cluster_name = 'MyCluster'
        self._session.set_credentials(self._access_key, self._secret_key)
        self._sts_client = self._session.create_client('sts', self._region)


class TestTokenGenerator(BaseTokenTest):
    @mock.patch.object(
        TokenGenerator,
        '_get_presigned_url',
        return_value='aHR0cHM6Ly9zdHMuYW1hem9uYXdzLmNvbS8=',
    )
    def test_token_no_padding(self, mock_presigned_url):
        generator = TokenGenerator(self._sts_client)
        tok = generator.get_token(self._cluster_name)
        self.assertTrue('=' not in tok)


class TestGetTokenCommand(BaseTokenTest):
    def test_get_expiration_time(self):
        cmd = GetTokenCommand(self._session)
        timestamp = cmd.get_expiration_time()
        try:
            datetime.strptime(timestamp, '%Y-%m-%dT%H:%M:%SZ')
        except ValueError:
            raise ValueError(
                "Incorrect data format, should be %Y-%m-%dT%H:%M:%SZ"
            )
